﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Text.RegularExpressions;
using FLY.Thick.RemoteHistory;
using System.ComponentModel;

namespace FLY.FeedbackRenZiJia.Common
{

    public class FlyData_FeedbackHeat : IFlyData
    {
        /// <summary>
        /// 开始测量时间！！！
        /// </summary>
        public DateTime Time { get; set; }
        /// <summary>
        /// 结束测量时间
        /// </summary>
        public DateTime EndTime { get; set; }


        /// <summary>
        /// 方向
        /// </summary>
        public Misc.DIRECTION Direction;

        /// <summary>
        /// 复位区号
        /// </summary>
        public int OrgBoltNo = 1;

        /// <summary>
        /// 旋转架旋转角度
        /// </summary>
        public double RAngle = 350;

        /// <summary>
        /// 膜距离
        /// </summary>
        public double FilmLength = 24;

        /// <summary>
        /// 旋转时间
        /// </summary>
        public TimeSpan RotatePeriod;

        /// <summary>
        /// 旋转次数
        /// </summary>
        public int RotateCnt;
        /// <summary>
        /// 偏移， 反方向为 Offset， 正方向为 -Offset
        /// </summary>
        public double Offset = 0;

        /// <summary>
        /// 厚度数据,  大小 NBolts
        /// </summary>
        public int[] Thicks;

        /// <summary>
        /// 开始改变加热时间
        /// </summary>
        public DateTime HTime;
        /// <summary>
        /// TODO 删除 加热数据编号，每次加热改变，编号++，这样可以不用比较加热是否相同
        /// </summary>
        public int HNo = -1;
        /// <summary>
        /// 加热数据,100%, 大小 ChannelCnt
        /// </summary>
        public int[] Heats;

        public bool IsStable
        {
            get
            {
                if ((int)Stability > 0)
                    return true;
                else
                    return false;
            }
        }
        /// <summary>
        /// 稳定状态
        /// </summary>
        public STABILITY Stability = STABILITY.IDLE;

        public string GetHeader()
        {
            string header = "测量时间,结束时间,旋转方向,旋转时间s,旋转次数,复位区号,旋转角度°,膜距离m";
            for (int i = 0; i < Thicks.Count(); i++)
            {
                header += ",厚度" + (i + 1).ToString();
            }
            header += ",加热时间,加热编号,加热起效,稳定性";
            for (int i = 0; i < Heats.Count(); i++)
            {
                header += ",加热" + (i + 1).ToString();
            }

            return header;
        }
        public override string ToString()
        {
            string str;
            str = Time.ToString();//测量时间
            str += "," + EndTime.ToString();//结束时间
            str += "," + Direction.ToString();//旋转方向
            str += "," + ((int)(RotatePeriod.TotalSeconds)).ToString();//旋转时间s
            str += "," + RotateCnt.ToString();//旋转次数

            str += "," + OrgBoltNo.ToString();//复位区号
            str += "," + RAngle.ToString("F1");//旋转角度°
            str += "," + FilmLength.ToString("F1");//膜距离m

            for (int i = 0; i < Thicks.Count(); i++)//厚度
            {
                if (Misc.MyBase.ISVALIDATA(Thicks[i]))
                    str += "," + (Thicks[i] / 100.0).ToString();
                else
                    str += ",";
            }

            str += "," + HTime.ToString();//加热时间

            str += "," + HNo.ToString();//加热编号

            if (Stability == STABILITY.ERROR_NO_ARRIVE)//加热起效
                str += "," + false.ToString();
            else
                str += "," + true.ToString();

            str += "," + Stability.ToString();//稳定性

            for (int i = 0; i < Heats.Count(); i++)//加热量
            {
                str += "," + Heats[i].ToString();
            }
            return str;
        }
        public bool TryParse(string header, string str)
        {
            string[] items = header.Split(new char[] { ',' });
            if (items.Length < 14)
                return false;

            int thicks_cnt = 0;//厚度的数量
            int heats_cnt = 0;//加热的数量
            int thicks1st_idx = -1;//第1个厚度 序号
            int heats1st_idx = -1;//第1个加热 序号
            //搜索 , 厚度
            Regex r = new Regex(@"厚度\d+");
            Regex r2 = new Regex(@"加热\d+");

            for (int i = 0; i < items.Count(); i++)
            {
                string s = items[i];
                if (r.IsMatch(s))
                {
                    if (thicks1st_idx == -1)
                        thicks1st_idx = i;
                    thicks_cnt++;
                }
                else if (r2.IsMatch(s))
                {
                    if (heats1st_idx == -1)
                        heats1st_idx = i;
                    heats_cnt++;
                }
            }

            if ((thicks_cnt == 0) || (heats_cnt == 0) || (thicks1st_idx == -1) || (heats1st_idx == -1))
            {
                //异常
                return false;
            }

            int len = items.Length;
            items = str.Split(new char[] { ',' });
            if ((items.Length) != len)
                return false;

            int idx = thicks1st_idx - 8;
            DateTime time;
            if (!DateTime.TryParse(items[idx], out time))
                return false;
            Time = time;
            idx += 1;

            if (!DateTime.TryParse(items[idx], out time))
                return false;
            EndTime = time;
            idx += 1;

            if (!Enum.TryParse<Misc.DIRECTION>(items[idx], out Direction))
                return false;
            idx += 1;

            int rotate_s;
            if (!int.TryParse(items[idx], out rotate_s))
                return false;
            RotatePeriod = TimeSpan.FromSeconds(rotate_s);
            idx += 1;

            if (!int.TryParse(items[idx], out RotateCnt))
                return false;
            idx += 1;


            if (!int.TryParse(items[idx], out OrgBoltNo))
                return false;
            idx += 1;

            if (!double.TryParse(items[idx], out RAngle))
                return false;
            idx += 1;

            if (!double.TryParse(items[idx], out FilmLength))
                return false;
            idx += 1;

            Thicks = new int[thicks_cnt];
            idx = thicks1st_idx;
            for (int i = 0; i < thicks_cnt; i++)
            {
                string s = items[idx];
                if (string.IsNullOrEmpty(s))
                {
                    Thicks[i] = Misc.MyBase.NULL_VALUE;

                }
                else
                {
                    double n;
                    if (double.TryParse(s, out n))
                        Thicks[i] = (int)(n * 100);
                    else
                        return false;
                }
                idx++;
            }



            idx = heats1st_idx - 4;
            if (!DateTime.TryParse(items[idx], out HTime))
                return false;
            idx += 1;

            if (!int.TryParse(items[idx], out HNo))
                return false;
            idx += 1;

            //加热起效
            idx += 1;

            if (!Enum.TryParse<STABILITY>(items[idx], out Stability))
                return false;
            idx += 1;

            Heats = new int[heats_cnt];
            idx = heats1st_idx;
            for (int i = 0; i < heats_cnt; i++)
            {
                string s = items[idx];
                if (string.IsNullOrEmpty(s))
                {
                    Heats[i] = Misc.MyBase.NULL_VALUE;
                }
                else
                {
                    int n;
                    if (int.TryParse(s, out n))
                        Heats[i] = n;
                    else
                        return false;
                }
                idx++;
            }

            return true;
        }



        public byte[] ToBytes()
        {
            List<byte> buf = new List<byte>();
            #region 测厚
            buf.AddRange(BitConverter.GetBytes(Time.Ticks));//8
            buf.AddRange(BitConverter.GetBytes(EndTime.Ticks));//8
            buf.AddRange(BitConverter.GetBytes((int)Direction));//4
            buf.AddRange(BitConverter.GetBytes(RotatePeriod.Ticks));//8
            buf.AddRange(BitConverter.GetBytes(RotateCnt));//4

            buf.AddRange(BitConverter.GetBytes(OrgBoltNo));//4
            buf.AddRange(BitConverter.GetBytes(RAngle));//8
            buf.AddRange(BitConverter.GetBytes(FilmLength));//8

            #region 厚度
            buf.AddRange(BitConverter.GetBytes(Thicks.Count()));//4
            for (int i = 0; i < Thicks.Count(); i++)
                buf.AddRange(BitConverter.GetBytes(Thicks[i]));//4

            #endregion

            #endregion
            #region 加热
            buf.AddRange(BitConverter.GetBytes(HTime.Ticks));//8
            buf.AddRange(BitConverter.GetBytes(HNo));//4
            buf.AddRange(BitConverter.GetBytes(Heats.Count()));//4
            for (int i = 0; i < Heats.Count(); i++)
                buf.AddRange(BitConverter.GetBytes(Heats[i]));//4
            #endregion

            #region 判断
            buf.AddRange(BitConverter.GetBytes((int)Stability));//4
            #endregion
            return buf.ToArray();
        }
        public bool TryParse(byte[] value)
        {
            int cnt;
            return TryParse(value, 0, out cnt);
        }
        public bool TryParse(byte[] value, int index, out int cnt)
        {
            cnt = 8 + 8 + 4 + 8 + 4;
            cnt += 4 + 8 + 8;
            if (value.Length < (index + cnt))
                return false;

            int idx = index;
            #region 测厚
            try
            {
                Time = new DateTime(BitConverter.ToInt64(value, idx));
            }
            catch
            {
                Time = DateTime.MinValue;
            }
            idx += 8;

            try
            {
                EndTime = new DateTime(BitConverter.ToInt64(value, idx));
            }
            catch
            {
                EndTime = DateTime.MinValue;
            }
            idx += 8;


            Direction = (Misc.DIRECTION)BitConverter.ToInt32(value, idx);
            idx += 4;


            try
            {
                RotatePeriod = new TimeSpan(BitConverter.ToInt64(value, idx));
            }
            catch
            {
                RotatePeriod = TimeSpan.MinValue;
            }
            idx += 8;

            RotateCnt = BitConverter.ToInt32(value, idx);
            idx += 4;


            OrgBoltNo = BitConverter.ToInt32(value, idx);
            idx += 4;

            RAngle = BitConverter.ToDouble(value, idx);
            idx += 8;

            FilmLength = BitConverter.ToDouble(value, idx);
            idx += 8;



            #region 厚度
            cnt += 4;
            if (value.Length < (index + cnt))
                return false;
            int len = BitConverter.ToInt32(value, idx);
            idx += 4;

            cnt += len * 4;
            if (value.Length < (index + cnt))
                return false;
            Thicks = new int[len];
            for (int i = 0; i < len; i++)
            {
                Thicks[i] = BitConverter.ToInt32(value, idx);
                idx += 4;
            }

            #endregion


            #endregion
            #region 加热
            cnt += 8 + 4 + 4;
            if (value.Length < (index + cnt))
                return false;

            try
            {
                HTime = new DateTime(BitConverter.ToInt64(value, idx));
            }
            catch
            {
                HTime = DateTime.MinValue;
            }
            idx += 8;

            HNo = BitConverter.ToInt32(value, idx);
            idx += 4;

            len = BitConverter.ToInt32(value, idx);
            idx += 4;

            cnt += len * 4;
            if (value.Length < (index + cnt))
                return false;

            Heats = new int[len];
            for (int i = 0; i < Heats.Count(); i++)
            {
                Heats[i] = BitConverter.ToInt32(value, idx);
                idx += 4;
            }
            #endregion
            return true;
        }
    }


    /// <summary>
    /// 稳定性
    /// </summary>
    public enum STABILITY
    {
        /// <summary>
        /// 待定，不知道！！
        /// </summary>
        IDLE = 0,
        /// <summary>
        /// 加热点，还没到达测厚仪。  其它状态都是加热已经起效了，顶多不稳定而已。
        /// </summary>
        ERROR_NO_ARRIVE = -1,
        /// <summary>
        /// 旋转次数太少，位置不能确定
        /// </summary>
        ERROR_POSITION_NOTSURE = -2,
        /// <summary>
        /// 一副数据，Sigma > 15% ： 不稳定，变化太大，超出控制范围 
        /// </summary>
        ERROR_SIGMA_OVERSIZE = -3,
        /// <summary>
        /// 连续两幅图旋转时间变化 > 10%：不稳定，旋转速度波动大，测量不准确
        /// </summary>
        ERROR_ROTATE_CHANGED = -4,
        /// <summary>
        /// 连续两幅图厚度均值变化 > 10%: 不稳定，厚度波动大
        /// </summary>
        ERROR_THICK_CHANGED = -5,





        /// <summary>
        /// 不稳定，不相关
        /// </summary>
        ERROR_CORREL = -6,

        /// <summary>
        /// 加热与厚度差，相关性很高
        /// </summary>
        OK_HEAT_AND_THICK_CORREL = 3,
        /// <summary>
        /// 加热为0，加热与厚度差，相关性很高
        /// </summary>
        OK_CORREL = 2,
    }






}
