﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Net;

using FLY.Thick.Blowing.Client;
using FLY.Thick.Base.Client;
using ThickTcpUiInWindow;
using System.ComponentModel;
using FLY.Thick.Blowing.IService;
using System.Collections.ObjectModel;

namespace FLY.Thick.Blowing.UI.Fix.Client
{
    /// <summary>
    /// Page_Blowing.xaml 的交互逻辑
    /// </summary>
    public partial class Page_Blowing : Page, INotifyPropertyChanged
    {
        BlowingFixServiceClient mRenZiJiaService;
        BlowingDetectServiceClient mBDetect;

        private bool isBufListLoading = false;
        /// <summary>
        /// 数据加载中
        /// </summary>
        public bool IsBufListLoading
        {
            get { return isBufListLoading; }
            set
            {
                if (isBufListLoading != value)
                {
                    isBufListLoading = value;
                    NotifyPropertyChanged("IsBufListLoading");
                }
            }
        }

        private int firstsignbm = -1;
        /// <summary>
        /// 信号列表第一个记录序号
        /// </summary>
        public int FirstSignBM
        {
            get { return firstsignbm; }
            protected set
            {
                if (firstsignbm != value)
                {
                    firstsignbm = value;
                    NotifyPropertyChanged("FirstSignBM");
                }
            }
        }
        private int lastsignbm = -1;
        /// <summary>
        /// 信号列表最后一个记录序号
        /// </summary>
        public int LastSignBM
        {
            get { return lastsignbm; }
            protected set
            {
                if (lastsignbm != value)
                {
                    lastsignbm = value;
                    NotifyPropertyChanged("LastSignBM");
                }
            }
        }
        ObservableCollection<SignData> mSignList = new ObservableCollection<SignData>();
        ObservableCollection<BoltMapCell> mBoltMap = new ObservableCollection<BoltMapCell>();


        public Page_Blowing()
        {
            InitializeComponent();
        }
        
        public void Init(IPEndPoint serverep) 
        {

            mRenZiJiaService = new BlowingFixServiceClient();
            mBDetect = new BlowingDetectServiceClient();

            FObjBase.FObjSys.Current.Connect_to_Another_OBJSys(
                serverep, mRenZiJiaService.ID, mBDetect.ID);

            this.grid_blowing.DataContext = mRenZiJiaService;
            this.grid_bDetect.DataContext = mBDetect;
            this.stackpanel_bmRangeCurr.DataContext = this;
            
            button_download.DataContext = this;
            gridFlows.ItemsSource = mSignList;

            UpdateMap();
            gridMap.ItemsSource = mBoltMap;

            mRenZiJiaService.PropertyChanged += MRenZiJiaService_PropertyChanged;

        }
        void UpdateMap()
        {
            mBoltMap.Clear();
            foreach (BoltMapCell m in mRenZiJiaService.Map)
            {
                mBoltMap.Add(m);
            }
        }
        private void MRenZiJiaService_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Map")
            {
                UpdateMap();
            }
        }
        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
        }


        private void Page_Unloaded(object sender, RoutedEventArgs e)
        {
            mRenZiJiaService.Dispose();
            mBDetect.Dispose();
        }
        
        bool CheckValid(out string message)
        {          
            //检查参数是否合法
            if (mBDetect.RAngle < 100 || mBDetect.RAngle > 360)
            {
                message = "旋转总角度";
                return false;
            }

            if ((mBDetect.DefaultRPeriod.TotalMinutes < 3) || (mBDetect.DefaultRPeriod.TotalMinutes > 16))
            {
                message = "旋转时间";
                return false;
            }

            if ((mBDetect.RollPerimeter < 100) || (mBDetect.RollPerimeter > 1000))
            {
                message = "辊周长";
                return false;
            }
            if ((mBDetect.FilmLength < 0) || (mBDetect.FilmLength > 100))
            {
                message = "膜距离";
                return false;
            }

            if (mBDetect.AccDecTime.TotalMilliseconds > 5000)
            {
                message = "加减速时间";
                return false;
            }

            if (mBDetect.LimitSignTime.TotalSeconds > 30)
            {
                message = "撞信号总时间";
                return false;
            }


            if (mRenZiJiaService.ChannelCnt < 20 || mRenZiJiaService.ChannelCnt > 160)
            {
                message = "加热通道数";
                return false;
            }
            if (mRenZiJiaService.BPC < 1 || mRenZiJiaService.BPC > 4)
            {
                message = "分区数/加热通道数";
                return false;
            }
            if (mRenZiJiaService.OrgBoltNo < 1 || mRenZiJiaService.OrgBoltNo > (mRenZiJiaService.ChannelCnt * mRenZiJiaService.BPC))
            {
                message = "复位区号";
                return false;
            }

            if ((mRenZiJiaService.FLRange < 1) || (mRenZiJiaService.FLRange > 20))
            {
                message = "查找范围";
                return false;
            }

            if (mRenZiJiaService.EPCType == RenZiJiaFixEPCType.EPCA10)
            {
                if (mRenZiJiaService.SampleConsume.TotalSeconds < 1 || mRenZiJiaService.SampleConsume.TotalSeconds > 30)
                {
                    message = "温修时间";
                    return false;
                }
                if (mRenZiJiaService.SampleInterval.TotalMinutes <= 1 || mRenZiJiaService.SampleInterval.TotalMinutes > 120)
                {
                    message = "温修间隔";
                    return false;
                }
                if (mRenZiJiaService.BackEdgeWait.TotalSeconds < 1 || mRenZiJiaService.BackEdgeWait.TotalSeconds > 30)
                {
                    message = "边界等待";
                    return false;
                }
            }
            message = "成功";
            return true;
        }
        private void button_apply_click(object sender, RoutedEventArgs e)
        {
            if (Window_Password.Authorize(1))
            {
                string message;
                bool isvalid = CheckValid(out message);
                if (!isvalid)
                { 
                    //异常出错
                    FLY.ControlLibrary.Window_WarningTip.Show("参数出错",message + "异常");
                }
                else
                {
                    mBDetect.Apply();
                    mRenZiJiaService.Apply();

                    FLY.ControlLibrary.Window_Tip.Show("应用成功",
                        null,
                        TimeSpan.FromSeconds(2));
                }
            }
        }

        private void button_cal_click(object sender, RoutedEventArgs e)
        {
            mRenZiJiaService.CalFilmLength();
        }
        protected void NotifyPropertyChanged(string propertyname)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyname));
        }
        public event PropertyChangedEventHandler PropertyChanged;

        private void button_download_Click(object sender, RoutedEventArgs e)
        {
            if (IsBufListLoading)
                return;

            IsBufListLoading = true;
            mBDetect.GetSignList(
                (AsyncState, retData) =>
                {
                    GetSignListReponse reponse = retData as GetSignListReponse;
                    //reponse.firstbm

                    IsBufListLoading = false;

                    FirstSignBM = reponse.firstbm;
                    LastSignBM = reponse.lastbm;
                    mSignList.Clear();
                    //反转顺序
                    if (reponse.datas != null)
                    {
                        for (int i = 0; i < reponse.datas.Count(); i++)
                        {
                            int idx = reponse.datas.Count() - 1 - i;
                            mSignList.Add(reponse.datas[idx]);
                        }
                    }
                    
                }, null);
        }
    }
    public class BoltLengthMultiConverter : IMultiValueConverter 
    {

        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            //FilmVelocity
            //RenZiJiaPeriod
            //NBolts
            if (values.Count() != 3) 
            {
                return null;
            }
            if ((values[0] is double) && (values[1] is TimeSpan) && (values[2] is int))
            {
                double FilmVelocity = (double)values[0];
                TimeSpan RenZiJiaPeriod = (TimeSpan)values[1];
                int NBolts = (int)values[2];
                return RenZiJiaPeriod.TotalMinutes / NBolts * FilmVelocity;
            }
            return null;

        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
