﻿
using FLY.ControlLibrary;
using FLY.Thick.BulkDataModule;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThickTcpUiInWindow.UIModule;

namespace FLY.Thick.Blowing.UI.Fix.Client.UIModule
{
    /// <summary>
    /// ScanGraphCircular.xaml 的交互逻辑
    /// </summary>
    public partial class ScanGraphCircular : UserControl
    {
        ScanGraphCircularParam graphparam;
        public ScanGraphCircular()
        {
            InitializeComponent();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="id"></param>
        public void Init(int id)
        {
            //查找参数
            graphparam = ScanGraphCircularParams.Current.Items.Find(p => p.ID == id);
            if (graphparam == null)
            {
                graphparam = new ScanGraphCircularParam() { ID = id };
                ScanGraphCircularParams.Current.Items.Add(graphparam);
                ScanGraphCircularParams.Current.Save();
            }

            //获取设备
            TDGage gage = Application.Current.Properties["Gage"] as TDGage;


            GraphScanCircular graph = graphscan;

            graph.DataContext = graphparam;
            graph.SetBinding(GraphScanCircular.TitleProperty, new Binding("Title"));
            graph.SetBinding(Grid.ColumnProperty, new Binding("Column"));
            graph.SetBinding(Grid.RowProperty, new Binding("Row"));
            graph.SetBinding(Grid.ColumnSpanProperty, new Binding("ColumnSpan"));
            graph.SetBinding(Grid.RowSpanProperty, new Binding("RowSpan"));
            graph.SetBinding(GraphScanCircular.BoltNoWithManual1stProperty, new Binding("BoltNoWithManual1st"));
            graph.SetBinding(GraphScanCircular.ManualCntProperty, new Binding("ManualCnt"));
            graph.SetBinding(GraphScanCircular.MPIsRightProperty, new Binding("MPIsRight"));

            //TODO
            graph.SetBinding(GraphScanCircular.BoltCntProperty, new Binding("NBolts") { Source = gage.mRenZiJiaService });
            graph.SetBinding(GraphScanCircular.OrgBoltNoProperty, new Binding("OrgBoltNo") { Source = gage.mRenZiJiaService });
            
            graph.SetBinding(GraphScanCircular.TargetProperty, new Binding("Target") { Source = gage.mDynArea });
            graph.SetBinding(GraphScanCircular.AlarmProperty, new Binding("Alarm") { Source = gage.mDynArea });




            //bulkdata 数据绑定！！！
            BulkDataServiceClientAdv.BulkDataGraphFrame bulkdata_client = new BulkDataServiceClientAdv.BulkDataGraphFrame();
            //graphparam 绑定到 bulkdataclient
            Misc.BindingOperations.SetBinding(graphparam, "BM", bulkdata_client, "BM");
            Misc.BindingOperations.SetBinding(graphparam, "Mix", bulkdata_client, "Mix");
            graph.SetBinding(GraphScanCircular.Title2Property, new Binding("Info") { Source = bulkdata_client });
            graph.DataSource = bulkdata_client.Frame;


            graph.ShowSettingDialogEvent += new EventHandler(delegate (object sender, EventArgs e)
            {
                ScanGraphCircularConfig window = new ScanGraphCircularConfig();
                window.Init((sender as GraphScanCircular).DataContext as ScanGraphCircularParam);
                window.Owner = FLY.ControlLibrary.COMMON.GetWindow(graph);
                window.ShowDialog();
            });

            gage.mBulkDataClient.Add(bulkdata_client);
        }
    }
    public class UIModule_ScanGraphCircular : FLY.UI.Module.IUIModule
    {
        /// <summary>
        /// 控件标题
        /// 它的值取决于culture
        /// </summary>
        public string Title
        {
            get
            {
                return Application.Current.FindResource("strScanGraphCircular") as string;
            }
        }

        /// <summary>
        /// 控件
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetComponent(int id)
        {
            ScanGraphCircular graph = new ScanGraphCircular();
            graph.Init(id);
            return graph;
        }

        /// <summary>
        /// 控件缩略图，用于编辑界面时，大致看看
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetThumbnail(int id)
        {
            return new System.Windows.Controls.Grid();
        }


        /// <summary>
        /// 给出全部控件ID, 控件自行删除没有的参数
        /// </summary>
        /// <param name="IDs"></param>
        public void MatchParam(int[] IDs)
        {
            ScanGraphCircularParams.Current.MatchParam(IDs);
        }
    }
    public class ScanGraphCircularParam : UIModuleParam
    {
        private string title = "环形扫描图";
        /// <summary>
        /// 标题
        /// </summary>
        public string Title
        {
            get
            {
                return title;
            }
            set
            {
                if (title != value)
                {
                    title = value;
                    NotifyPropertyChanged("Title");
                }
            }
        }
        private int mix = 1;
        /// <summary>
        /// 混合数
        /// </summary>
        public int Mix
        {
            get
            {
                return mix;
            }
            set
            {
                if (mix != value)
                {
                    mix = value;
                    NotifyPropertyChanged("Mix");
                }
            }
        }
        private int bm = -1;
        /// <summary>
        /// 记录点
        /// </summary>
        public int BM
        {
            get
            {
                return bm;
            }
            set
            {
                if (bm != value)
                {
                    bm = value;
                    NotifyPropertyChanged("BM");
                }
            }
        }
        private int boltNoWithManual1st = 1;
        /// <summary>
        /// 模头手动调节的螺丝对应的分区号
        /// </summary>
        public int BoltNoWithManual1st
        {
            get
            {
                return boltNoWithManual1st;
            }
            set
            {
                if (boltNoWithManual1st != value)
                {
                    boltNoWithManual1st = value;
                    NotifyPropertyChanged("BoltNoWithManual1st");
                }
            }
        }
        private int manualCnt = 8;
        /// <summary>
        /// 模头手动调节的螺丝个数
        /// </summary>
        public int ManualCnt
        {
            get
            {
                return manualCnt;
            }
            set
            {
                if (manualCnt != value)
                {
                    manualCnt = value;
                    NotifyPropertyChanged("ManualCnt");
                }
            }
        }
        private bool mPIsRight = true;
        /// <summary>
        /// 探头在右边
        /// </summary>
        public bool MPIsRight
        {
            get
            {
                return mPIsRight;
            }
            set
            {
                if (mPIsRight != value)
                {
                    mPIsRight = value;
                    NotifyPropertyChanged("MPIsRight");
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string[] GetSavePropertyNames()
        {
            List<string> names = new List<string>(base.GetSavePropertyNames());
            names.AddRange(new string[]{
                "Title",
                "BM","Mix",
                "BoltNoWithManual1st",
                "ManualCnt",
                "MPIsRight"
            });
            return names.ToArray();
        }
    }

    /// <summary>
    /// 全部扫描图的参数
    /// </summary>
    public class ScanGraphCircularParams : UIModuleParams<ScanGraphCircularParam>
    {
        static ScanGraphCircularParams()
        {
            Misc.SaveToXmlHepler.Regist(typeof(ScanGraphCircularParams));
        }
        public ScanGraphCircularParams()
        {
            string path = System.IO.Path.Combine(FLYLayout.BasePath, "scangraphcircular.xml");
            FilePath = path;
        }

        static ScanGraphCircularParams current = null;
        public static ScanGraphCircularParams Current
        {
            get
            {
                if (current == null)
                {
                    current = new ScanGraphCircularParams();
                    current.Load();
                }
                return current;
            }
        }
    }
}
