﻿using FLY.Thick.Base.OBJ_INTERFACE;
using FLY.Thick.Blowing.Common;
using FLY.Thick.Blowing.IService;
using FLY.Thick.Blowing.OBJ_INTERFACE;
using FObjBase;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace FLY.Thick.Blowing.Client
{
    public class BlowingFixProfileService : FObj, IBlowingFixProfileService
    {
        IFConn mConn;
        UInt32 mServerID;

        public BlowingFixProfileService()
        {
            mServerID = FLY.Thick.Blowing.OBJ_INTERFACE.OBJ_INTERFACE_ID.BFPROFILE_ID;
        }
        public BlowingFixProfileService(UInt32 id)
        {
            mServerID = id;
        }

        #region IProfileService  接口
        BlowingFixProfileParam param = new BlowingFixProfileParam();
        public BlowingFixProfileParam Param
        {
            get
            {
                return param;
            }
        }


        /// <summary>
        /// 应用 &amp; 保存
        /// </summary>
        public void Apply()
        {
            CurrObjSys.SetValueEx(mConn, mServerID, ID,
                BLOWINGFIX_PROFILE_OBJ_INTERFACE.SET_PARAMS,
                Param.ToBytes());
        }



        /// <summary>
        /// 获取产品列表, 返回类型为 List&lt;string&gt;
        /// </summary>
        /// <param name="AsyncDelegate"></param>
        /// <param name="AsyncState"></param>
        public void GetList(AsyncCBHandler AsyncDelegate, object AsyncState)
        {
            CurrObjSys.CallFunctionEx(mConn, mServerID, ID,
                BLOWINGFIX_PROFILE_OBJ_INTERFACE.CALL_GETLIST,
                null,
                AsyncDelegate, AsyncState);
        }

        /// <summary>
        /// 删除指定产品
        /// </summary>
        /// <param name="productname"></param>
        public void Del(string productname)
        {
            Pack_String p = new Pack_String();
            p.data = productname;
            CurrObjSys.CallFunctionEx(mConn, mServerID, ID,
                BLOWINGFIX_PROFILE_OBJ_INTERFACE.CALL_DEL, p.ToBytes());
        }

        /// <summary>
        /// 读取指定产品,返回类型为 ProfileParam
        /// </summary>
        /// <param name="productname"></param>
        /// <param name="AsyncDelegate"></param>
        /// <param name="AsyncState"></param>
        public void Read(string productname, AsyncCBHandler AsyncDelegate, object AsyncState)
        {
            CurrObjSys.CallFunctionEx(mConn, mServerID, ID,
                BLOWINGFIX_PROFILE_OBJ_INTERFACE.CALL_READ,
                new Pack_String() { data = productname }.ToBytes(),
                AsyncDelegate, AsyncState);
        }
        #endregion



        public override void Dispose()
        {
            CurrObjSys.ObjRemove(
                this, mConn);
        }
        public override void ConnectNotify(IFConn from)
        {
            mConn = from;
            if (from.IsConnected)
            {
                //获取所有数据，设置推送
                CurrObjSys.GetValueEx(
                    mConn, mServerID, ID,
                    BLOWINGFIX_PROFILE_OBJ_INTERFACE.GET_PARAMS);

                CurrObjSys.SenseConfigEx(
                    mConn, mServerID, ID,
                    0xffffffff,
                    SENSE_CONFIG.ADD);
            }
        }
        public override void PushGetValue(IFConn from, uint srcid, ushort memid, byte[] infodata)
        {
            switch (memid)
            {
                case BLOWINGFIX_PROFILE_OBJ_INTERFACE.GET_PARAMS:
                    {
                        BlowingFixProfileParam p = new BlowingFixProfileParam();

                        if (!p.TryParse(infodata))
                            return;
                        Param.TryParse(infodata);

                    }
                    break;
            }
        }
        public override void PushInfo(IFConn from, uint srcid, ushort infoid, byte[] infodata)
        {

            PushGetValue(from, srcid, infoid, infodata);


        }
        public override void PushCallFunction(IFConn from, uint srcid, uint magic, ushort funcid, byte[] retdata, object AsyncDelegate, object AsyncState)
        {
            switch (funcid)
            {
                case BLOWINGFIX_PROFILE_OBJ_INTERFACE.CALL_GETLIST:
                    {
                        Pack_StringList p = new Pack_StringList();
                        if (!p.TryParse(retdata))
                            return;
                        ((AsyncCBHandler)AsyncDelegate)(AsyncState, p.list);
                    }
                    break;
                case BLOWINGFIX_PROFILE_OBJ_INTERFACE.CALL_READ:
                    {
                        BlowingFixProfileParam p = new BlowingFixProfileParam();
                        if (!p.TryParse(retdata))
                            return;
                        ((AsyncCBHandler)AsyncDelegate)(AsyncState, p);
                    }
                    break;
            }
        }
    }
}
