﻿using FLY.Thick.Blowing.IService;
using FLY.Thick.Blowing.OBJ_INTERFACE;
using FObjBase;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace FLY.Thick.Blowing.Client
{
    public class BlowingServiceClient : FObj, IBlowing
    {
        protected IFConn mConn;
        protected UInt32 mServerID;

        private bool isconnected = false;
        public bool IsConnected
        {
            get { return isconnected; }
            set
            {
                if (isconnected != value)
                {
                    isconnected = value;
                    NotifyPropertyChanged("IsConnected");
                }
            }
        }

        #region IBlowing 成员变量
        #region 分区设定
        private int channelcnt = 44;
        /// <summary>
        /// 加热通道数
        /// </summary>
        public int ChannelCnt
        {
            get
            {
                return channelcnt;
            }
            set
            {
                if (channelcnt != value)
                {
                    channelcnt = value;
                    NotifyPropertyChanged("ChannelCnt");
                    NotifyPropertyChanged("NBolts");
                }
            }
        }

        private int bpc = 2;
        /// <summary>
        /// 分区数/加热通道数
        /// </summary>
        public int BPC
        {
            get
            {
                return bpc;
            }
            set
            {
                if (bpc != value)
                {
                    bpc = value;
                    NotifyPropertyChanged("BPC");
                    NotifyPropertyChanged("NBolts");
                }
            }
        }


        /// <summary>
        /// 风环螺丝数
        /// </summary>
        public int NBolts
        {
            get { return ChannelCnt * BPC; }
        }




        private int orgboltno = 1;
        public int OrgBoltNo
        {
            get
            {
                return orgboltno;
            }
            set
            {
                if (orgboltno != value)
                {
                    orgboltno = value;
                    NotifyPropertyChanged("OrgBoltNo");
                }
            }
        }

        private bool isUsedMap = false;
        /// <summary>
        /// 使用分区表
        /// </summary>
        public bool IsUsedMap
        {
            get { return isUsedMap; }
            set {
                if (isUsedMap != value)
                {
                    isUsedMap = value;
                    NotifyPropertyChanged("IsUsedMap");
                }
            }
        }

        private List<BoltMapCell> map = new List<BoltMapCell>();
        /// <summary>
        /// 分区表
        /// </summary>
        public List<BoltMapCell> Map
        {
            get { return map; }
            set {
                map = value;
                NotifyPropertyChanged("Map");
            }
        }
        #endregion


        #endregion

        /// <summary>
        /// 每次有新数据时候推送。 修改参数，不会推送
        /// </summary>
        public event RenZiJiaDataEventHandler DataEvent;

        public BlowingServiceClient()
        {
            mServerID = FLY.Thick.Blowing.OBJ_INTERFACE.OBJ_INTERFACE_ID.RENZIJIA_ID;
        }

        #region 功能
        public virtual void Apply()
        {
            CurrObjSys.SetValueEx(
                mConn, mServerID, ID,
                BLOWING_OBJ_INTERFACE.SET_PARAMS,
                new BLOWING_OBJ_INTERFACE.Pack_Params()
                {
                    channelcnt = ChannelCnt,
                    bpc = BPC,
                    orgBolt = OrgBoltNo
                }.ToBytes());

            CurrObjSys.SetValueEx(
                mConn, mServerID, ID,
                BLOWING_OBJ_INTERFACE.SET_MAP,
                new BLOWING_OBJ_INTERFACE.Pack_Map()
                {
                    IsUsedMap = IsUsedMap,
                    Map = Map
                }.ToBytes());
        }

        #endregion


        #region INotifyPropertyChanged 成员
        protected void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion

        public override void Dispose()
        {
            CurrObjSys.ObjRemove(
                this, mConn);
        }
        public override void ConnectNotify(IFConn from)
        {
            mConn = from;
            IsConnected = from.IsConnected;
            if (from.IsConnected)
            {
                //获取所有数据，设置推送
                CurrObjSys.GetValueEx(
                    mConn, mServerID, ID,
                    BLOWING_OBJ_INTERFACE.GET_PARAMS);

                CurrObjSys.GetValueEx(
                    mConn, mServerID, ID,
                    BLOWING_OBJ_INTERFACE.GET_MAP);

                CurrObjSys.SenseConfigEx(
                    mConn, mServerID, ID, 0xffffffff,
                    SENSE_CONFIG.ADD);

            }
        }
        public override void PushGetValue(IFConn from, uint srcid, ushort memid, byte[] infodata)
        {
            switch (memid)
            {
                case BLOWING_OBJ_INTERFACE.GET_PARAMS:
                    {
                        BLOWING_OBJ_INTERFACE.Pack_Params p = new BLOWING_OBJ_INTERFACE.Pack_Params();
                        if (!p.TryParse(infodata))
                            break;

                        ChannelCnt = p.channelcnt;//总分区数，从1开始
                        BPC = p.bpc;
                        OrgBoltNo = p.orgBolt;
                    }
                    break;
                case BLOWING_OBJ_INTERFACE.GET_MAP:
                    {
                        BLOWING_OBJ_INTERFACE.Pack_Map p = new BLOWING_OBJ_INTERFACE.Pack_Map();
                        if (!p.TryParse(infodata))
                            break;
                        IsUsedMap = p.IsUsedMap;
                        Map = p.Map;
                    }
                    break;
            }
        }
        public override void PushInfo(IFConn from, uint srcid, ushort infoid, byte[] infodata)
        {
            switch (infoid)
            {
                case BLOWING_OBJ_INTERFACE.PUSH_DATA:
                    {
                        RenZiJiaDataEventArgs e = new RenZiJiaDataEventArgs();
                        if (!e.TryParse(infodata))
                            return;
                        DataEvent?.Invoke(this, e);
                    }
                    break;
                default:
                    PushGetValue(from, srcid, infoid, infodata);
                    break;
            }
        }

       
    }
}
