﻿using Misc;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace FLY.Thick.Blowing.Common
{
    public class BlowingFixProfileParam : INotifyPropertyChanged
    {

        #region 正常运行参数
        /// <summary>
        /// 产品名称
        /// </summary>
        public string PName { get; set; }= "test_pname";

        /// <summary>
        /// 目标值
        /// </summary>
        public int Target { get; set; } = 3000;

        /// <summary>
        /// 上下限
        /// </summary>
        public int Alarm { get; set; } = 100;

        /// <summary>
        /// 斜率补偿
        /// </summary>
        public double Comp { get; set; } = 1;


        #endregion
        #region 吹膜定点解方程用
        /// <summary>
        /// 使能定点解方程
        /// </summary>
        public bool SolveEnable { get; set; } = false;

        /// <summary>
        /// 膜宽 单位 mm
        /// </summary>
        public int FilmWidth { get; set; } = 1500;

        /// <summary>
        /// 探头所在膜的横向位置 单位 mm
        /// </summary>
        public int FilmPosH { get; set; } = 70;

        #endregion
        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion

        #region IPack 成员

        public virtual byte[] ToBytes()
        {
            List<byte> buf = new List<byte>();
            byte[] bs;

            #region 正常运行参数
            bs = Misc.Converter.StringToBytes(PName);
            buf.AddRange(BitConverter.GetBytes(bs.Length));
            buf.AddRange(bs);
            
            buf.AddRange(BitConverter.GetBytes(Target));
            buf.AddRange(BitConverter.GetBytes(Alarm));

            buf.AddRange(BitConverter.GetBytes(Comp));
            #endregion

            #region 吹膜定点解方程用
            buf.AddRange(BitConverter.GetBytes(SolveEnable));
            buf.AddRange(BitConverter.GetBytes(FilmWidth));
            buf.AddRange(BitConverter.GetBytes(FilmPosH));
            #endregion
            return buf.ToArray();
        }

        public virtual bool TryParse(byte[] value)
        {
            int cnt = 0;
            return TryParse(value, 0, out cnt);
        }

        public virtual bool TryParse(byte[] value, int index, out int cnt)
        {

            cnt = 0;
            int idx = index;

            #region 正常运行参数
            cnt += 4 + 4 + 4 + 8;
            if (value.Length < index + cnt)
                return false;

            int len = BitConverter.ToInt32(value, idx);
            idx += 4;
            cnt += len;
            if (value.Length < index + cnt)
                return false;

            PName = Misc.Converter.BytesToString(value, idx, len);
            idx += len;

            Target = BitConverter.ToInt32(value, idx);
            idx += 4;

            Alarm = BitConverter.ToInt32(value, idx);
            idx += 4;

            Comp = BitConverter.ToDouble(value, idx);
            idx += 8;

            #endregion
            #region 吹膜定点解方程用
            cnt += 1 + 4 + 4;
            if (value.Length < index + cnt)
                return false;
            SolveEnable = BitConverter.ToBoolean(value, idx);
            idx += 1;
            FilmWidth = BitConverter.ToInt32(value, idx);
            idx += 4;
            FilmPosH = BitConverter.ToInt32(value, idx);
            idx += 4;
            #endregion

            return true;
        }

        #endregion

        public void CopyTo(BlowingFixProfileParam param)
        {
            param.PName = PName;
            param.Target = Target;
            param.Alarm = Alarm;
            param.Comp = Comp;
            param.SolveEnable = SolveEnable;
            param.FilmWidth = FilmWidth;
            param.FilmPosH = FilmPosH;
        }
    }
}
