﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace FLY.Thick.Blowing.IService
{
    /// <summary>
    /// 基本吹膜系统
    /// </summary>
    public interface IBlowing : INotifyPropertyChanged
    {
        #region 分区设定
        /// <summary>
        /// 加热通道数
        /// </summary>
        int ChannelCnt { get; set; }

        /// <summary>
        /// 分区数/加热通道数
        /// </summary>
        int BPC { get; set; }

        /// <summary>
        /// 人字架总分区数，从1开始数
        /// </summary>
        int NBolts { get; }

        /// <summary>
        /// 人字架复位时, 探头对应的分区位， 也就是 0° 对应的分区位
        /// </summary>
        int OrgBoltNo { get; set; }

        /// <summary>
        /// 使用分区表
        /// </summary>
        bool IsUsedMap { get; set; }
        /// <summary>
        /// 分区表,必须保证不能为null
        /// </summary>
        List<BoltMapCell> Map { get; set; } 
        #endregion

        /// <summary>
        /// 参数应用
        /// </summary>
        void Apply();

        /// <summary>
        /// 每次有新数据时候推送。 修改参数，不会推送
        /// </summary>
        event RenZiJiaDataEventHandler DataEvent;
    }

    /// <summary>
    /// 扫描一次，推送
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate void RenZiJiaDataEventHandler(object sender, RenZiJiaDataEventArgs e);

    /// <summary>
    /// 扫描一次，推送的数据
    /// </summary>
    public class RenZiJiaDataEventArgs : EventArgs
    {
        /// <summary>
        /// 测量数据开始时间
        /// </summary>
        public DateTime Time;

        /// <summary>
        /// 测量数据结束时间
        /// </summary>
        public DateTime EndTime;

        /// <summary>
        /// 旋转方向
        /// </summary>
        public Misc.DIRECTION Direction;

        /// <summary>
        /// 旋转1周的时间
        /// </summary>
        public TimeSpan RotatePeriod;

        /// <summary>
        /// 旋转次数
        /// </summary>
        public int RotationCnt;

        /// <summary>
        /// 1幅数据
        /// </summary>
        public int[] Frame;


        #region IPack 成员

        public byte[] ToBytes()
        {
            List<byte> buf = new List<byte>();
            buf.AddRange(BitConverter.GetBytes(Time.Ticks));
            buf.AddRange(BitConverter.GetBytes(EndTime.Ticks));
            buf.AddRange(BitConverter.GetBytes((int)Direction));
            buf.AddRange(BitConverter.GetBytes(RotatePeriod.Ticks));

            buf.AddRange(BitConverter.GetBytes(RotationCnt));

            buf.AddRange(BitConverter.GetBytes(Frame.Length));
            for (int i = 0; i < Frame.Count(); i++)
            {
                buf.AddRange(BitConverter.GetBytes(Frame[i]));
            }
            return buf.ToArray();
        }

        public bool TryParse(byte[] value)
        {
            int cnt;
            return TryParse(value, 0, out cnt);
        }

        public bool TryParse(byte[] value, int index, out int cnt)
        {
            cnt = 8 + 8 + 4 + 8 + 4 * 2;
            if (value.Length < (index + cnt))
                return false;
            int idx = index;
            try
            {
                Time = new DateTime(BitConverter.ToInt64(value, idx));
                idx += 8;
            }
            catch
            {
                return false;
            }

            try
            {
                EndTime = new DateTime(BitConverter.ToInt64(value, idx));
                idx += 8;
            }
            catch
            {
                return false;
            }

            Direction = (Misc.DIRECTION)BitConverter.ToInt32(value, idx);
            idx += 4;

            try
            {
                RotatePeriod = new TimeSpan(BitConverter.ToInt64(value, idx));
                idx += 8;
            }
            catch
            {
                return false;
            }


            RotationCnt = BitConverter.ToInt32(value, idx);
            idx += 4;

            int len = BitConverter.ToInt32(value, idx);
            idx += 4;

            cnt += len * 4;
            if (value.Length < (index + cnt))
                return false;
            if (len == 0)
                Frame = null;
            else
            {
                Frame = new int[len];
                for (int i = 0; i < len; i++)
                {
                    Frame[i] = BitConverter.ToInt32(value, idx);
                    idx += 4;
                }
            }
            return true;
        }
        #endregion
    }

    public class BoltMapCell
    {
        /// <summary>
        /// 旧位置
        /// </summary>
        public int OldNo { get; set; }
        /// <summary>
        /// 新位置
        /// </summary>

        public int NewNo { get; set; }

        /// <summary>
        /// 全复制
        /// </summary>
        /// <returns></returns>
        public BoltMapCell Clone()
        {
            return new BoltMapCell() { NewNo = NewNo, OldNo = OldNo };
        }
        
        public byte[] ToBytes()
        {
            List<byte> buf = new List<byte>();
            buf.AddRange(BitConverter.GetBytes(OldNo));//4
            buf.AddRange(BitConverter.GetBytes(NewNo));//4
            return buf.ToArray();
        }

        /// <summary>
        /// 8个byte
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public bool TryParse(byte[] value)
        {
            return TryParse(value, 0);
        }

        /// <summary>
        /// 8个byte
        /// </summary>
        /// <param name="value"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        public bool TryParse(byte[] value, int index)
        {
            int cnt = 4 + 4;

            if (value.Length < (index + cnt))
                return false;

            int idx = index;

            OldNo = BitConverter.ToInt32(value, idx);
            idx += 4;
            NewNo = BitConverter.ToInt32(value, idx);
            idx += 4;
            return true;
        }
    }
}
