﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FObjBase;
using FLY.Thick.Blowing.IService;

namespace FLY.Thick.Blowing.OBJ_INTERFACE
{
    public class BLOWINGFIX_OBJ_INTERFACE:BLOWING_OBJ_INTERFACE
    {
        #region Pack
        public class Pack_ParamsExt : IPack
        {
            /// <summary>
            /// 膜长查找范围
            /// </summary>
            public double filmLenRange;

            #region IPack 成员

            public byte[] ToBytes()
            {
                List<byte> buf = new List<byte>();
                buf.AddRange(BitConverter.GetBytes(filmLenRange));
                return buf.ToArray();
            }

            
            public bool TryParse(byte[] value)
            {
                if (value.Length < 8)
                    return false;
                int idx = 0;
                
                filmLenRange = BitConverter.ToDouble(value, idx);
                idx += 8;
                return true;
            }

            #endregion
        }
        public class Pack_CalR : IPack 
        {
            public double filmLenRange;//膜长查找范围
            #region IPack 成员

            public byte[] ToBytes()
            {
                List<byte> buf = new List<byte>();
                buf.AddRange(BitConverter.GetBytes(filmLenRange));
                return buf.ToArray();
            }

            public bool TryParse(byte[] value)
            {
                if (value.Length <8)
                    return false;
                int idx = 0;
                filmLenRange = BitConverter.ToDouble(value, idx);
                idx += 8;


                return true;
            }

            #endregion
        }
        public class Pack_Relevency : IPack 
        {
            public double max_relevency;
            
            public double max_filmLength;//m

            public string calstate;

            #region IPack 成员

            public byte[] ToBytes()
            {
                List<byte> buf = new List<byte>();
                buf.AddRange(BitConverter.GetBytes(max_relevency));
                buf.AddRange(BitConverter.GetBytes(max_filmLength));
                byte[] c = FObjBase.Converter.StringToBytes(calstate);
                buf.AddRange(BitConverter.GetBytes(c.Length));
                buf.AddRange(c);
                return buf.ToArray();
            }

            public bool TryParse(byte[] value)
            {
                if (value.Length < 20)
                    return false;
                int idx = 0;
                max_relevency = BitConverter.ToDouble(value, idx);
                idx += 8;
                max_filmLength = BitConverter.ToDouble(value, idx);
                idx += 8;
                int cl = BitConverter.ToInt32(value, idx);
                idx += 4;
                if (value.Length < (idx + cl))
                    return false;
                byte[] c = new byte[cl];
                Array.Copy(value,idx, c, 0, cl);
                calstate = FObjBase.Converter.BytesToString(c);
                return true;
            }

            #endregion
        }

        public class Pack_EPC : IPack
        {
            public RenZiJiaFixEPCType epctype;
            public TimeSpan sampleconsume;
            public TimeSpan sampleinterval;
            public TimeSpan backedgewait;
            public bool isBtnSelfHold;
            #region IPack 成员

            public byte[] ToBytes()
            {
                List<byte> buf = new List<byte>();
                buf.AddRange(BitConverter.GetBytes((int)epctype));
                buf.AddRange(BitConverter.GetBytes(sampleconsume.Ticks));
                buf.AddRange(BitConverter.GetBytes(sampleinterval.Ticks));
                buf.AddRange(BitConverter.GetBytes(backedgewait.Ticks));
                buf.AddRange(BitConverter.GetBytes(isBtnSelfHold));
                return buf.ToArray();
            }

            public bool TryParse(byte[] value)
            {
                if (value.Length < 4 + 8 +8 +8+1)
                    return false;
                int idx = 0;
                epctype = (RenZiJiaFixEPCType)BitConverter.ToInt32(value, idx);
                idx += 4;
                
                sampleconsume = new TimeSpan(BitConverter.ToInt64(value, idx));
                idx += 8;

                sampleinterval = new TimeSpan(BitConverter.ToInt64(value, idx));
                idx += 8;

                backedgewait = new TimeSpan(BitConverter.ToInt64(value, idx));
                idx += 8;

                isBtnSelfHold = BitConverter.ToBoolean(value, idx);
                idx += 1;

                return true;
            }

            #endregion
        }
        public class Pack_EPCState : IPack
        {
            public TimeSpan sampletimer;
            public bool isrunning;
            public int samplead;
            #region IPack 成员

            public byte[] ToBytes()
            {
                List<byte> buf = new List<byte>();
                buf.AddRange(BitConverter.GetBytes(sampletimer.Ticks));
                buf.AddRange(BitConverter.GetBytes(isrunning));
                buf.AddRange(BitConverter.GetBytes(samplead));
                return buf.ToArray();
            }

            public bool TryParse(byte[] value)
            {
                if (value.Length < 8 + 1+4)
                    return false;
                int idx = 0;
                sampletimer = new TimeSpan(BitConverter.ToInt64(value, idx));
                idx += 8;

                isrunning = BitConverter.ToBoolean(value, idx);
                idx += 1;

                samplead = BitConverter.ToInt32(value, idx);
                idx += 4;
                return true;
            }

            #endregion
        }

        #endregion
        #region GetValue
        /// <summary>
        /// Pack_Params
        /// </summary>
        public const UInt16 GET_PARAMSEXT = 3;

        /// <summary>
        /// Pack_Relevency
        /// </summary>
        public const UInt16 GET_R = 4;

        /// <summary>
        /// Pack_EPC
        /// </summary>
        public const UInt16 GET_EPC = 5;
        /// <summary>
        /// Pack_EPCState
        /// </summary>
        public const UInt16 GET_EPCSTATE = 6;

        #endregion
        #region SetValue
        /// <summary>
        /// Pack_Params
        /// </summary>
        public const UInt16 SET_PARAMSEXT = 3;
        /// <summary>
        /// Pack_EPC
        /// </summary>
        public const UInt16 SET_EPC = 5;

        #endregion
        #region CallFunction
        /// <summary>
        /// Pack_CalR
        /// </summary>
        public const UInt16 CALL_CALR = 1;

        public const UInt16 CALL_EPCSTART = 2;

        public const UInt16 CALL_EPCSTOP = 3;

        #endregion
        #region PushInfo
        /// <summary>
        /// Pack_Params
        /// </summary>
        public const UInt16 PUSH_PARAMSEXT = 3;

        /// <summary>
        /// Pack_Relevency
        /// </summary>
        public const UInt16 PUSH_R = 4;

        /// <summary>
        /// Pack_EPC
        /// </summary>
        public const UInt16 PUSH_EPC = 5;

        /// <summary>
        /// Pack_EPCState
        /// </summary>
        public const UInt16 PUSH_EPCSTATE = 6;

        #endregion
    }
}
