﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.IO;
using System.Xml.Serialization;
using System.Threading;

using Misc;
using FlyADBase;
using FLY.Thick.BulkDataModule;
using FLY.Thick.Base.Common;
using FLY.Thick.Base.IService;
using FLY.Thick.Base.Server;
using FLY.Thick.Blowing.IService;
using MathNet.Numerics.LinearAlgebra.Double;
using FObjBase;

namespace FLY.Thick.Blowing.Server
{

    public class GM_BlowingFix : GM_Base, IBlowingFixService, ISaveToXml
    {


        #region 延时操作，markno
        const int MARKNO_SAVE = 1;
        const int MARKNO_CALFRAME = 2;
        #endregion

        /// <summary>
        /// 人字架角度 &amp; 膜走带位置检测
        /// </summary>
        public BlowingDetect mPDetect;
        private CalFilmLen mCalFilmLen;
        #region ISaveToXml
        public string[] GetSavePropertyNames()
        {
            return new string[]{
                "ChannelCnt",
                "BPC",
                "OrgBoltNo",
                "FLRange",
                "EPCType",
                "SampleConsume",
                "SampleInterval",
                "BackEdgeWait",
                "IsBtnSelfHold",
                "IsUsedMap",
                "Map"
            };
        }
        #endregion

        #region IRenZiJiaService

        #region 分区设定

        private int channelcnt = 44;
        /// <summary>
        /// 加热通道数
        /// </summary>
        public int ChannelCnt 
        {
            get {
                return channelcnt;
            }
            set {
                if (channelcnt != value) 
                {
                    channelcnt = value;
                    NotifyPropertyChanged("ChannelCnt");
                    NotifyPropertyChanged("NBolts");
                }
            }
        }

        private int bpc=2;
        /// <summary>
        /// 分区数/加热通道数
        /// </summary>
        public int BPC 
        {
            get 
            {
                return bpc;
            }
            set {
                if (bpc != value) 
                {
                    bpc = value;
                    NotifyPropertyChanged("BPC");
                    NotifyPropertyChanged("NBolts");
                }
            }
        }

        
        /// <summary>
        /// 风环螺丝数
        /// </summary>
        public int NBolts
        {
            get { return ChannelCnt * BPC; }
        }


        private int orgboltno;
        /// <summary>
        /// 复位时，探头测量的位置对应分区号
        /// </summary>
        public int OrgBoltNo 
        {
            get { return orgboltno; }
            set {
                
                if (orgboltno != value) 
                {
                    orgboltno = value;
                    NotifyPropertyChanged("OrgBoltNo");
                }
            }
        }


        /// <summary>
        /// 第1个螺丝号
        /// </summary>
        public int BoltNo1st=1;

        private bool isUsedMap = false;
        /// <summary>
        /// 使用分区表
        /// </summary>
        public bool IsUsedMap {
            get {
                return isUsedMap;
            }
            set {
                if (isUsedMap != value)
                {
                    isUsedMap = value;
                    NotifyPropertyChanged("IsUsedMap");
                }
            }
        }

        private List<BoltMapCell> map = new List<BoltMapCell>();
        /// <summary>
        /// 分区表,必须保证不能为null
        /// </summary>
        public List<BoltMapCell> Map
        {
            get { return map; }
            set {
                //检测是否不一样
                if (value == null)
                    return;
                if (checkNoSame_map(value))
                {
                    map = value;
                    if(checkParamErr_map())
                        NotifyPropertyChanged("Map");
                }
            }
        }
        bool checkNoSame_map(List<BoltMapCell> value)
        {
            if (map.Count() != value.Count())
                return true;
            for (int i = 0; i < map.Count(); i++)
            {
                if (map[i].NewNo != value[i].NewNo)
                    return true;
                if (map[i].OldNo != value[i].OldNo)
                    return true;
            }
            return false;
        }
        #endregion

        #region 膜长 计算

        private double flrange;
        public double FLRange
        {
            get { return flrange; }
            set {
                if (flrange != value) 
                {
                    flrange = value;
                    NotifyPropertyChanged("FLRange");
                }
            }
        }

        public void CalFilmLength() 
        {
            mCalFilmLen.Start();
        }
        #region 计算结果
        private double maxR;
        public double MaxR
        {
            get { return maxR; }
            protected set
            {
                if (maxR != value)
                {
                    maxR = value;
                    NotifyPropertyChanged("MaxR");
                }
            }
        }


        private double maxRFilmLength;//最大相识度对应的膜长度
        /// <summary>
        /// 最大相识度对应的膜长度
        /// </summary>
        public double MaxRFilmLength
        {
            get { return maxRFilmLength; }
            protected set
            {
                if (maxRFilmLength != value)
                {
                    maxRFilmLength = value;
                    NotifyPropertyChanged("MaxRFilmLength");
                }
            }
        }
        private string calstate="null";
        /// <summary>
        /// 计算膜长状态
        /// </summary>
        public string CalState
        {
            get { return calstate; }
            protected set
            {
                if (calstate != value)
                {
                    calstate = value;
                    NotifyPropertyChanged("CalState");
                }
            }
        }
        #endregion

        #endregion


        #region IRenZiJiaFixService

        /// <summary>
        /// 追边开始按钮自锁
        /// </summary>
        public bool IsBtnSelfHold 
        {
            get {
                return mEPCA10.IsBtnSelfHold;
            }
            set {
                mEPCA10.IsBtnSelfHold = value;
            }
        }

        RenZiJiaFixEPCType epctype = RenZiJiaFixEPCType.Null;
        public RenZiJiaFixEPCType EPCType 
        {
            get 
            { 
                return epctype; 
            }
            set
            {
                if (epctype != value) 
                {
                    epctype = value;
                    NotifyPropertyChanged("EPCType");
                }
            }
        }
        /// <summary>
        /// 采样时间， 默认2s
        /// </summary>
        public TimeSpan SampleConsume
        {
            get { return mEPCA10.SampleConsume; }
            set
            {
                mEPCA10.SampleConsume = value;
            }
        }

        /// <summary>
        /// 温修间隔，默认45分钟
        /// </summary>
        public TimeSpan SampleInterval
        {
            get { return mEPCA10.SampleInterval; }
            set
            {
                mEPCA10.SampleInterval = value;
            }
        }

        /// <summary>
        /// 当前温修计时，大于温修间隔就会去温修
        /// </summary>
        public TimeSpan SampleTimer 
        {
            get {
                return mEPCA10.SampleTimer;
            }
        }



        /// <summary>
        /// 回到边界后，再等待一段时间，那就肯定找到边界了。 默认5s
        /// </summary>
        public TimeSpan BackEdgeWait 
        {
            get {
                return mEPCA10.BackEdgeWait;
            }
            set {
                mEPCA10.BackEdgeWait = value;
            }
        }

        /// <summary>
        /// 采样得到的样品AD
        /// </summary>
        public int SampleAD 
        {
            get
            {
                return mEPCA10.SampleAD;
            }           
        }
        /// <summary>
        /// 追边运行中！
        /// </summary>
        public bool EPCIsRunning 
        {
            get {
                return mEPCA10.IsRunning;
            }
        }

        #endregion

        public void Apply() 
        {
            
            Save();
        }
        public event RenZiJiaDataEventHandler DataEvent;
        #endregion

        #region EPC
        public delegate void EPCSampledEventHandler(int ad);

        class EPCA10:INotifyPropertyChanged
        {
            /// <summary>
            /// EPC-A10 警报 对应输入位
            /// </summary>
            const int in_epc_alarm_bit = 12;
            /// <summary>
            /// EPC启动 对应输入位
            /// </summary>
            const int in_epc_enable_bit = 6;
            /// <summary>
            /// EPC启动状态 对应输出位
            /// </summary>
            const int out_epc_isrunning_bit = 1;
            /// <summary>
            /// 控制EPC-A10 到温修点 对应输出位
            /// </summary>
            const int out_epc_sample_bit = 2;
            #region public成员

            public bool Enable { get; set; } = false;

            /// <summary>
            /// 温修时间，默认2秒
            /// </summary>
            public TimeSpan SampleConsume { get; set; } = TimeSpan.FromSeconds(2);


            /// <summary>
            /// 温修间隔，默认45分钟
            /// </summary>
            public TimeSpan SampleInterval { get; set; } = TimeSpan.FromMinutes(45);


            /// <summary>
            /// 当前温修计时，大于温修间隔就会去温修
            /// </summary>
            public TimeSpan SampleTimer { get; set; }= TimeSpan.Zero;

            /// <summary>
            /// 回到边界，再等待1段时间
            /// </summary>
            public TimeSpan BackEdgeWait { get; set; } = TimeSpan.FromSeconds(5);



            /// <summary>
            /// 追边开始按钮自锁
            /// </summary>
            public bool IsBtnSelfHold { get; set; } = true;



            /// <summary>
            /// 采样得到的样品AD
            /// </summary>
            public int SampleAD { get; set; } = -1;


            /// <summary>
            /// 追边运行中！
            /// </summary>
            public bool IsRunning { get; set; } = false;

            public event EPCSampledEventHandler Sampled;
            #endregion





            /// <summary>
            /// 采样开始时间
            /// </summary>
            /// <returns></returns>
            DateTime SampleBegin;

            /// <summary>
            /// 回去膜边开始时间
            /// </summary>
            DateTime GoBackBegin;

            /// <summary>
            /// 去采样开始时间
            /// </summary>
            DateTime GoSampleBegin;

            /// <summary>
            /// 去采样消耗的时间
            /// </summary>
            TimeSpan GoSampleConsume;



            IFlyAD mFlyAD;

            List<int> sampledata = new List<int>();
            TimeSpan sampleConsumeTimer = TimeSpan.Zero;
            public EPCA10(IFlyAD flyad)
            {
                mFlyAD = flyad;
                mFlyAD.IStatusChangedEvent += new IStatusChangedEventHandler(mFlyAD_IStatusChangedEvent);
                mFlyAD.TimeGridEvent += new FlyADBase.TimeGridEventHandler(mFlyAD_TimeGridEvent);
                this.PropertyChanged += EPCA10_PropertyChanged;
            }

            private void EPCA10_PropertyChanged(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Enable")
                {
                    //TODO, 错的！！！！
                    if(!Enable)
                        Stop();
                }
            }

            void mFlyAD_TimeGridEvent(object sender, FlyADBase.TimeGridEventArgs e)
            {
                if (!Enable)
                    return;

                if (State != STATE.SAMPLE)
                    return;

                TimeSpan ts = e.Ts;//准确

                DateTime dt = e.Time;//准确
                int[] data = e.Data;

                if (dt < SampleBegin)
                    return;

                sampledata.AddRange(data);
                sampleConsumeTimer += TimeSpan.FromTicks(ts.Ticks * data.Length);

                if (sampleConsumeTimer >= SampleConsume)
                {
                    int ad = Misc.MyMath.Avg(sampledata.ToArray());
                    if (Misc.MyBase.ISVALIDATA(ad))
                    {
                        SampleAD = ad;
                        if (Sampled != null)
                            Sampled(SampleAD);
                    }
                    else 
                    {
                        SampleAD = -1;
                    }
                    FinishSample();
                }
            }

            void mFlyAD_IStatusChangedEvent(object sender, IStatusChangedEventArgs e)
            {
                if (!Enable)
                    return;

                if (Misc.MyBase.CHECKBIT(e.IChanged, in_epc_enable_bit - 1))
                {
                    if (IsBtnSelfHold)
                    {
                        if (!Misc.MyBase.CHECKBIT(e.IStatus, in_epc_enable_bit - 1))
                        {
                            if (!IsRunning)
                            {
                                Start();
                            }
                        }
                        else
                        {
                            if (IsRunning)
                            {
                                Stop();
                            }
                        }
                    }
                    else 
                    {
                        if (!Misc.MyBase.CHECKBIT(e.IStatus, in_epc_enable_bit - 1))
                        {
                            if (!IsRunning)
                            {
                                Start();
                            }
                            else
                            {
                                Stop();
                            }
                        }
                    }
                }
            }
            enum STATE
            {
                /// <summary>
                /// 没动作
                /// </summary>
                STOP,
                /// <summary>
                /// 准备去温修
                /// </summary>
                START,
                /// <summary>
                /// 去温修过程中，记录耗费的时间 
                /// </summary>
                WAITFOR_SAMPLE,
                /// <summary>
                /// 已经到达温修点，温修中,采样 N秒
                /// </summary>
                SAMPLE,
                /// <summary>
                /// 等待 耗费的时间+N秒
                /// </summary>
                WAITFOR_EDGE,
                /// <summary>
                /// 测量中，等待N分钟，下次会进入START
                /// </summary>
                WAITFOR_NEXT,
            }
            /// <summary>
            /// 追边状态
            /// </summary>
            STATE State = STATE.STOP;

            public void Start()
            {
                if (!Enable)
                    return;
                State = STATE.START;
                SetRunning();
            }
            public void Stop()
            {
                if (!Enable)
                    return;
                State = STATE.STOP;
                ClearRunning();
                GoBack();
            }
            void SetRunning()
            {
                IsRunning = true;
                mFlyAD.SetOutput(
                    (UInt16)Misc.MyBase.BIT(out_epc_isrunning_bit - 1),
                    (UInt16)~Misc.MyBase.BIT(out_epc_isrunning_bit - 1));
            }
            void ClearRunning()
            {
                IsRunning = false;
                mFlyAD.SetOutput(
                    (UInt16)Misc.MyBase.BIT(out_epc_isrunning_bit - 1),
                    (UInt16)Misc.MyBase.BIT(out_epc_isrunning_bit - 1));
            }
            bool CheckIsRunning() 
            {
                if (!Misc.MyBase.CHECKBIT(mFlyAD.OStatus, out_epc_isrunning_bit - 1))
                    return true;
                else
                    return false;
            }
            void GoSample()
            {
                mFlyAD.SetOutput(
                    (UInt16)Misc.MyBase.BIT(out_epc_sample_bit - 1),
                    (UInt16)~Misc.MyBase.BIT(out_epc_sample_bit - 1));

            }
            void GoBack()
            {
                mFlyAD.SetOutput(
                    (UInt16)Misc.MyBase.BIT(out_epc_sample_bit - 1),
                    (UInt16)Misc.MyBase.BIT(out_epc_sample_bit - 1));
            }
            bool IsAlarm()
            {
                if (!Misc.MyBase.CHECKBIT(mFlyAD.IStatus, in_epc_alarm_bit - 1))
                    return true;
                else
                    return false;
            }
            public bool IsSampling()
            {

                switch (State)
                {
                    case STATE.WAITFOR_SAMPLE:
                    case STATE.SAMPLE:
                    case STATE.WAITFOR_EDGE:
                        return true;
                    default:
                        return false;
                }
            }

            void FinishSample()
            {
                State = STATE.WAITFOR_EDGE;
                GoBack();
                GoBackBegin = DateTime.Now;
            }
            /// <summary>
            /// 应该放在完成一幅图时触发
            /// </summary>
            public void CheckToSample()
            {
                if (!Enable)
                    return;
                if (!IsRunning)
                    return;

                if(SampleTimer >= SampleInterval)
                {
                    Start();
                }
            }

            /// <summary>
            /// 0.1秒触发一次
            /// </summary>
            public void OnPoll()
            {
                if (!Enable)
                    return;
                
                switch (State)
                {
                    case STATE.STOP:
                        if (CheckIsRunning())
                            ClearRunning();//这是不正常的，灭了它
                        break;
                    case STATE.START:
                        {
                            sampledata.Clear();
                            SampleTimer = TimeSpan.Zero;
                            GoSample();
                            GoSampleBegin = DateTime.Now;
                            
                            State = STATE.WAITFOR_SAMPLE;
                        } break;
                    case STATE.WAITFOR_SAMPLE:
                        {
                            if (IsAlarm())
                            {
                                GoSampleConsume = DateTime.Now - GoSampleBegin;
                                //已经到达
                                State = STATE.SAMPLE;
                                SampleBegin = DateTime.Now;
                                sampleConsumeTimer = TimeSpan.Zero;
                            }
                        } break;
                    case STATE.SAMPLE:
                        {

                        } break;
                    case STATE.WAITFOR_EDGE:
                        {
                            TimeSpan ts = DateTime.Now - GoBackBegin;
                            if (ts >= GoSampleConsume + BackEdgeWait)
                            {
                                //完成
                                State = STATE.WAITFOR_NEXT;
                            }
                        } break;
                    case STATE.WAITFOR_NEXT:
                        {
                            if (SampleTimer < SampleInterval)
                            {
                                SampleTimer += TimeSpan.FromSeconds(1);
                            }
                        }break;
                }
            }

            public event PropertyChangedEventHandler PropertyChanged;

        }
        EPCA10 mEPCA10;
        #endregion

        #region 吹膜定点解方程用
        private bool solveenable = false;
        /// <summary>
        /// 使能定点解方程
        /// </summary>
        public bool SolveEnable
        {
            get { return solveenable; }
            set
            {
                if (solveenable != value)
                {
                    solveenable = value;
                    NotifyPropertyChanged("SolveEnable");
                }
            }
        }
        private int filmWidth = 1500;
        /// <summary>
        /// 膜宽 单位 mm
        /// </summary>
        public int FilmWidth
        {
            get { return filmWidth; }
            set
            {
                if (filmWidth != value)
                {
                    filmWidth = value;
                    NotifyPropertyChanged("FilmWidth");
                }
            }
        }

        private int filmPosH = 70;
        /// <summary>
        /// 探头所在膜的横向位置 单位 mm
        /// </summary>
        public int FilmPosH
        {
            get { return filmPosH; }
            set
            {
                if (filmPosH != value)
                {
                    filmPosH = value;
                    NotifyPropertyChanged("FilmPosH");
                }
            }
        }

        #endregion


        IBulkDataServiceAdd mBulkData;
        DynArea mDynArea;

        /// <summary>
        /// 当前正在处理的 ADList 里面的数据.No
        /// </summary>
        int mADListToDoNo;
        /// <summary>
        /// 处理放入filminfo 的数据.No
        /// </summary>
        int mADListToDoNo_ToFilmInfo;
        

        RList<ADCell> mADList;
        
        class ADCell 
        {
            /// <summary>
            /// 测量的时间
            /// </summary>
            public DateTime dt;
            /// <summary>
            /// 用于修正AB值，温修AD后，重新计算thick。
            /// </summary>
            public int ad=Misc.MyBase.NULL_VALUE;
            /// <summary>
            /// 上面的膜对应的角度 0~360°
            /// </summary>
            public double angle0;
            /// <summary>
            /// 下面的膜对应的角度 0~360°
            /// </summary>
            public double angle1;
            /// <summary>
            /// 人字架正向旋转
            /// </summary>
            public Misc.DIRECTION direction;
            /// <summary>
            /// 线速度
            /// </summary>
            public double velocity;
            /// <summary>
            /// 人字架旋转次数, 当为-1 时，没获取到数据，等待中！！
            /// </summary>
            public int rotationCnt;
            /// <summary>
            /// 恒速阶段
            /// </summary>
            public bool inCV;
            public override string ToString()
            {
                return dt.Minute.ToString() + ":" + (dt.Second + dt.Millisecond / 1000.0).ToString("N4") + " " +
                    "angle=(" + angle0.ToString("N1") + "," + angle1.ToString("N1") + ")";
            }
            public ADCell Clone() 
            {
                ADCell c = new ADCell()
                {
                    dt = dt,
                    ad = ad
                };
                return c;
            }
        }

        /// <summary>
        /// 1条方程
        /// </summary>
        class Formula
        {
            /// <summary>
            /// n元方程
            /// </summary>
            public int n;
            /// <summary>
            /// 方程 xi前面的 ai
            /// </summary>
            public double[] ai;
            /// <summary>
            /// 方程 y 值
            /// </summary>
            public double y;
            public Formula(int no_x)
            {
                int i;
                n = no_x;
                ai = new double[no_x];
                for (i = 0; i < n; i++) ai[i] = 0;
                y = 0;
            }
            public bool AddData(int idx1, int idx2, int thick)
            {
                if ((idx1 < 0) || (idx1 >= n)) return false;
                if ((idx2 < 0) || (idx2 >= n)) return false;
                ai[idx1]++;
                ai[idx2]++;
                y += thick;
                return true;
            }
        }
        class FRAME_INFO//一幅数据信息
        {
            
            public DateTime StartTime;//开始时间
            public DateTime EndTime;//结束时间
            public int NoStartAD;//在AD列表 处理的数据开始点
            public int NoEndAD;//在AD列表 处理的数据结束点
            public DIRECTION direction;//当前幅数据，方向
            public double velocity;//线速度 m/min
            public int rotationCnt;//人字架旋转次数
            /// <summary>
            /// 对应bulkdata 的数据, -1, 没有对应的bookmark，因为从来都没数据
            /// </summary>
            public int bookmark = -1;
            public int[] frame;//分区数据 N*long
            int frame_sum = 0;
            int frame_cnt = 0;

            static int virtual_nbolts = 88;
            static List<Formula> virtual_formulas = new List<Formula>();
            static void UpdateVirtualFormulas()
            {
                virtual_formulas.Clear();
                for (int i = 0; i < virtual_nbolts; i++)
                {
                    int boltindex1 = i;
                    int boltindex2 = i + 1;
                    if (boltindex2 >= virtual_nbolts)
                        boltindex2 = 0;

                    Formula fm = new Formula(virtual_nbolts);
                    fm.ai[boltindex1] = 1;
                    fm.ai[boltindex2] = -1;
                    virtual_formulas.Add(fm);
                }
            }
            static FRAME_INFO()
            {
                UpdateVirtualFormulas();
            }
            public void SetNBolt(int nbolts) 
            {
                frame = new int[nbolts];
                appeared = new bool[nbolts];
                for (int i = 0; i < nbolts; i++)
                {
                    ClearFrameValue(i);
                }
            }
            int index_last = -1;
            public void SetFrameValue(int index,int value,DateTime dt)
            {
                int nbolts = frame.Count();

                if (index >= nbolts)
                    return;//异常
                else if (index < 0)
                    return;//异常
                else if (!Misc.MyBase.ISVALIDATA(value))
                    return;//异常

                if (index_last != index)
                {
                    index_last = index;
                    HasChanged = true;
                    frame_cnt = 0;
                    frame_sum = 0;
                }
                    
                frame_sum += value;
                frame_cnt++;
                    
                frame[index] = frame_sum / frame_cnt;
                
            }
            void ClearFrameValue(int index) 
            {
                frame[index] = Misc.MyBase.NULL_VALUE;
                appeared[index] = false;
            }
            public void Clear() 
            {
                for (int i = 0; i < frame.Count(); i++)
                {
                    ClearFrameValue(i);
                }
                IsValid = true;
                HasChanged = true;
                index_last = -1;
                frame_cnt = 0;
                frame_sum = 0;
                mFormulas.Clear();
                last_solvedt = DateTime.MinValue;
                isTimeToSolve = false;
                canSolve = false;
            }
            
            public bool HasChanged = false;//数据被改变了！！！，是时候更新bulkdata
            public bool IsValid = true;//是合法数据
            /// <summary>
            /// 提交，完成一次操作。清除状态
            /// </summary>
            public void Submit() 
            {
                HasChanged = false;
            }
            public void CheckForValid() 
            {
                if (rotationCnt == 0)//旋转次数=0，肯定是不完整的数据
                {
                    HasChanged = true;
                    IsValid = false;
                    return;
                }
                //所有的数据都应该大于10
                for (int i = 0; i < frame.Count(); i++) 
                {
                    if (Misc.MyBase.ISVALIDATA(frame[i])) 
                    {
                        if (frame[i] < 1000)
                        {
                            HasChanged = true;
                            IsValid = false;
                            return;
                        }
                    }
                }
            }
            /// <summary>
            /// 上一次解方程的时间
            /// </summary>
            DateTime last_solvedt = DateTime.MinValue;
            /// <summary>
            /// 是时候解方程
            /// </summary>
            bool isTimeToSolve = false;
            /// <summary>
            /// 方程多了，能解方程
            /// </summary>
            bool canSolve = false;
            /// <summary>
            /// 每个区号，是否有出现
            /// </summary>
            bool[] appeared;
            /// <summary>
            /// 方程组
            /// </summary>
            List<Formula> mFormulas = new List<Formula>();
            public void SolveFormula()
            {
                if (!canSolve)//没有新方程，没法解
                    return;

                int nbolts = frame.Count();
                if (virtual_nbolts!= nbolts)
                {
                    virtual_nbolts = nbolts;
                    UpdateVirtualFormulas();
                }
                //TODO 能否解得方程检查
                int nrow = mFormulas.Count() + virtual_formulas.Count();
                
                //if (nrow < NBolts) return;

                //Console.WriteLine("Total get {0} formular", nrow);
                //转换为 DenseMatrix 与 DenseVector
                DenseMatrix matrixA = new DenseMatrix(nrow, nbolts);
                DenseMatrix matrixAT = new DenseMatrix(nbolts, nrow);
                DenseVector vectorB = new DenseVector(nrow);
                DenseVector vectorATB = new DenseVector(nbolts);

                int rowindex = 0;
                foreach (Formula fm in mFormulas)
                {
                    matrixA.SetRow(rowindex, fm.ai);
                    matrixAT.SetColumn(rowindex, fm.ai);
                    vectorB[rowindex] = fm.y;
                    rowindex++;
                }
                foreach (Formula fm in virtual_formulas)
                {
                    matrixA.SetRow(rowindex, fm.ai);
                    matrixAT.SetColumn(rowindex, fm.ai);
                    vectorB[rowindex] = fm.y;
                    rowindex++;
                }
                DenseMatrix ATA = (DenseMatrix)matrixA.TransposeThisAndMultiply(matrixA);
                DenseVector ATB = (DenseVector)matrixA.TransposeThisAndMultiply(vectorB);

                DenseVector X = (DenseVector)ATA.Solve(ATB);

                for (int i = 0; i < nbolts; i++)
                {
                    int d = (appeared[i]) ? (int)X[i] : Misc.MyBase.NULL_VALUE;
                    frame[i] = d;// (int)X[i];
                }
                HasChanged = true;

                isTimeToSolve = false;
                canSolve = false;
            }

            /// <summary>
            /// 检查是否需要更新
            /// </summary>
            public void CheckToSolveFormula()
            {
                if(isTimeToSolve)
                {
                    SolveFormula();
                }
            }
            /// <summary>
            /// 解方程方式更新 frame
            /// </summary>
            /// <param name="index0"></param>
            /// <param name="index1"></param>
            /// <param name="value"></param>
            /// <param name="dt"></param>
            public void SetFrameValue(int index0, int index1, int value, DateTime dt)
            {
                int nbolts = frame.Count();

                if (index0 >= nbolts)
                    return;//异常
                else if (index0 < 0)
                    return;//异常
                else if (index1 >= nbolts)
                    return;//异常
                else if (index1 < 0)
                    return;//异常
                else if (!Misc.MyBase.ISVALIDATA(value))
                    return;//异常

                

                Formula fm = new Formula(nbolts);
                fm.AddData(index0, index1, value);
                mFormulas.Add(fm);
                appeared[index0] = true;
                appeared[index1] = true;
                
            
                canSolve = true;

                //5秒解一次方程
                if ((index_last != index0) && ((dt - last_solvedt) >= TimeSpan.FromSeconds(5)))
                {
                    index_last = index0;
                    last_solvedt = dt;
                    isTimeToSolve = true;
                }
            }
        }
        RList<FRAME_INFO> mFrameInfoList;


        AD2ThickHandler AD2Thick;

        public event EPCSampledEventHandler EPCSampled 
        {
            add {
                mEPCA10.Sampled += value;
            }
            remove {
                mEPCA10.Sampled -= value;              
            }
        }

        void checkParamErr() 
        {
            if (ChannelCnt < 20 || ChannelCnt > 160)
            {
                ChannelCnt = 44;
            }
            
            if (BPC < 1 || BPC > 4)
            {
                BPC = 2;
            }

            if (OrgBoltNo < 1 || OrgBoltNo > NBolts)
            {
                OrgBoltNo = 1;
            }
            
            if ((FLRange < 1) || (FLRange > 20))
            {
                FLRange = 5;
            }

            checkParamErr_map();
        }

        /// <summary>
        /// 检测通过，返回true
        /// </summary>
        /// <returns></returns>
        bool checkParamErr_map()
        {
            bool map_err = false;
            bool g_o = false;
            bool g_n = false;
            //Map 检测
            //OldNo,NewNo不能 <1 >NBolts
            //Map OldNo 必须从小到大环形排列
            //Map NewNo 必须从小到大环形排列
            for (int i = 0; i < Map.Count(); i++)
            {
                if (Map[i].NewNo < 1 || Map[i].NewNo > NBolts)
                {
                    map_err = true;
                    break;
                }
                if (Map[i].OldNo < 1 || Map[i].OldNo > NBolts)
                {
                    map_err = true;
                    break;
                }

                int i_next = i + 1;
                if (i_next >= Map.Count())
                    i_next = 0;

                if (Map[i].OldNo >= Map[i_next].OldNo)
                {
                    if (!g_o)
                        g_o = true;
                    else
                    {
                        map_err = true;
                        break;
                    }

                }

                if (Map[i].NewNo >= Map[i_next].NewNo)
                {
                    if (!g_n)
                        g_n = true;
                    else
                    {
                        map_err = true;
                        break;
                    }

                }
            }
            if (map_err)
            {
                IsUsedMap = false;
                Map.Clear();
                NotifyPropertyChanged("Map");
                return false;
            }
            else
            {
                return true;
            }
        }

        static GM_BlowingFix()
        {
            Misc.SaveToXmlHepler.Regist(typeof(BoltMapCell));
        }
        public GM_BlowingFix(FlyAD7 flyad)
            :base(flyad)
        {
            GMState = CTRL_STATE.FIX;
            
            mPDetect = new BlowingDetect();
            mCalFilmLen = new CalFilmLen(this);
            mEPCA10 = new EPCA10(flyad);

            ChannelCnt = 44;//加热棒数量
            BPC = 2;//分区数量/加热棒数量
            OrgBoltNo = 1;//复位时，测厚仪对应的螺丝号

            FLRange = 5;//5m
            MaxR = -1;//最近第3幅数据 与 最近第2幅数据 在搜索后找到的最大相识度
            MaxRFilmLength = -1;

            EPCType = RenZiJiaFixEPCType.Null;

            mPDetect.Load();
            Load();


            //检测参数
            mPDetect.CheckParamErr();
            checkParamErr();

            mEPCA10.Enable = (EPCType == RenZiJiaFixEPCType.EPCA10);
            mEPCA10.PropertyChanged += new PropertyChangedEventHandler(mEPCA10_PropertyChanged);
            InitList();

            PropertyChanged += new PropertyChangedEventHandler(RenZiJiaFix_PropertyChanged);
            
            mPDetect.FilmInfoChangedEvent += new BlowingDetect.FilmInfoChangedEventHandler(mPDetect_FilmInfoChangedEvent);
            mPDetect.ClearEvent += new BlowingDetect.ClearEventHandler(mPDetect_ClearEvent);


            mFlyAD.TimeGridEvent += new FlyADBase.TimeGridEventHandler(flyad_TimeGridEvent);

            mPDetect.Init(flyad);
        }

        void mEPCA10_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if ((e.PropertyName == "SampleConsume") ||
                (e.PropertyName == "SampleInterval") ||
                (e.PropertyName == "SampleTimer") ||
                (e.PropertyName == "BackEdgeWait") ||
                (e.PropertyName == "IsBtnSelfHold"))
            {
                NotifyPropertyChanged(e.PropertyName);
            }
            else if (e.PropertyName == "IsRunning")
            {
                NotifyPropertyChanged("EPCIsRunning");
            }
        }



        public void EPCStart() 
        {
            if(EPCType == RenZiJiaFixEPCType.EPCA10)
                mEPCA10.Start();
        }

        public void EPCStop()
        {
            if (EPCType == RenZiJiaFixEPCType.EPCA10)
                mEPCA10.Stop();
        }

        void mPDetect_ClearEvent(object sender)
        {
            //没信号，清空所有数据！！！
        }
        /// <summary>
        /// 从1幅数据，获取头部，尾部，为于 ADList 的位置
        /// </summary>
        /// <param name="frameinfo"></param>
        /// <param name="index_start"></param>
        /// <param name="index_end"></param>
        /// <returns></returns>
        bool GetFilmInfoADIndex(FRAME_INFO frameinfo, out int index_start, out int index_end) 
        {
            index_start = mADList.No2Index(frameinfo.NoStartAD);
            
            index_end = mADList.No2Index(frameinfo.NoEndAD);
            
            if ((index_start < 0)||(index_end <0)||
                (index_start >= mADList.Count)||(index_end >= mADList.Count))
            {
                //数据已经不存在，删除
                return false;
            }
            return true;
        }


        void mPDetect_FilmInfoChangedEvent(object sender, BlowingDetect.FilmInfoChangedEventArgs e)
        {
            int idx;
            int ret = ADListFastSearchIndex(e.Time, out idx);
            if (ret < 0)//发送在过去
            {
                if (mADList.Count > 0)
                {
                    mADListToDoNo = mADList.Index2No(0);
                    mADListToDoNo_ToFilmInfo = mADListToDoNo;
                }
            }
            else if (ret == 0) 
            {
                mADListToDoNo = mADList.Index2No(idx);
                mADListToDoNo_ToFilmInfo = mADListToDoNo;
            }
            
        }

        void RenZiJiaFix_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "NBolts")
            {
                //所有数据删除
                mFrameInfoList.Clear();

            }
            else if (e.PropertyName == "EPCType")
            {
                mEPCA10.Enable = (EPCType == RenZiJiaFixEPCType.EPCA10);
            }
            else if (e.PropertyName == "OrgBoltNo")
            {
                //TODO 只处理3幅图
                int idx = mFrameInfoList.Count - 3;
                if (idx < 0)
                    idx = 0;

                if (idx >= mFrameInfoList.Count)
                    return;
                mADListToDoNo_ToFilmInfo = mFrameInfoList[idx].NoStartAD;
            }
            else if ((e.PropertyName == "FilmWidth") || (e.PropertyName == "FilmPosH") ||
                (e.PropertyName == "SolveEnable"))
            {
                //重新计算当前幅数据
                UpdateCurrFrame();
            }
        }

        public void Init(IBulkDataServiceAdd bulkdata, AD2ThickHandler func_ad2thick, DynArea dynarea )
        {
            AD2Thick = func_ad2thick;
            mBulkData = bulkdata;
            mDynArea = dynarea;

            mDynArea.NBolts = NBolts;
            mDynArea.FirstBoltNo = 1;

            mDynArea.DataBoltNoBegin = 1;
            mDynArea.DataBoltNoEnd = NBolts;
            mDynArea.ScanBoltNoBegin = 1;
            mDynArea.ScanBoltNoEnd = NBolts;

            Misc.BindingOperations.SetBinding(this, "FilmVelocity", mDynArea, "FilmVelocity");

            PropertyChanged += (s, e) =>
            {
                if (e.PropertyName == "NBolts")
                {
                    mBulkData.NBolts = NBolts;

                    mDynArea.NBolts = NBolts;
                    mDynArea.FirstBoltNo = 1;

                    mDynArea.DataBoltNoBegin = 1;
                    mDynArea.DataBoltNoEnd = NBolts;
                    mDynArea.ScanBoltNoBegin = 1;
                    mDynArea.ScanBoltNoEnd = NBolts;
                }
            };

            dynarea.PropertyChanged += new PropertyChangedEventHandler(dynarea_PropertyChanged);
            dynarea.SampleAD.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(SampleAD_CollectionChanged);
        }

        void SampleAD_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            //重新计算当前幅数据
            UpdateCurrFrame();
        }


        void dynarea_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            DynArea dynarea = sender as DynArea;
            if (e.PropertyName == "Comp")
            {
                //重新计算当前幅数据
                UpdateCurrFrame();
            }
        }

        void UpdateCurrFrame() 
        {
            FObjBase.PollModule.Current.Poll_JustOnce(
                new FObjBase.PollModule.PollHandler(delegate()
                {
                    if (mFrameInfoList.Count > 0) 
                    {
                        mADListToDoNo_ToFilmInfo = mFrameInfoList.Last().NoStartAD;
                    }

                }), this, MARKNO_CALFRAME);
        } 


        
        TimeSpan datOf100ms_ts = TimeSpan.Zero;
        List<int> datOf100ms = new List<int>();
        void flyad_TimeGridEvent(object sender, FlyADBase.TimeGridEventArgs e)
        {
            bool issample = false;
            if (EPCType == RenZiJiaFixEPCType.EPCA10)
            {
                if (mEPCA10.IsSampling())
                {
                    issample = true;
                }
            }

            TimeSpan ts = e.Ts;//准确
 
            DateTime dt = e.Time;//准确
            int[] data = e.Data;

            DateTime datOf100ms_dt = DateTime.MinValue;
            for(int i=0;i<data.Length;i++)
            {
                if (issample)//采样中，数据都是无效的
                {
                    datOf100ms.Add(Misc.MyBase.NULL_VALUE);
                }
                else if (data[i] > UInt16.MaxValue)
                {
                    datOf100ms.Add(Misc.MyBase.NULL_VALUE);
                }
                else if (data[i] < 0)
                {
                    datOf100ms.Add(Misc.MyBase.NULL_VALUE);
                }
                else
                {
                    datOf100ms.Add(data[i]);
                }

                datOf100ms_ts = datOf100ms_ts + ts;

                if (datOf100ms_dt == DateTime.MinValue)
                    datOf100ms_dt = dt;
                else 
                    datOf100ms_dt = datOf100ms_dt + ts;
                
                if (datOf100ms_ts.TotalMilliseconds>=100)
                {
                    datOf100ms_ts = datOf100ms_ts.Subtract(new TimeSpan(0, 0, 0, 0, 100));

                    int avgad = Misc.MyBase.NULL_VALUE;

                    if (datOf100ms.Count() > 0)
                    {
                        avgad = Misc.MyMath.Avg(datOf100ms.ToArray());
                        datOf100ms.Clear();
                    }
                    else if (mADList.Count() > 0)
                    {
                        avgad = mADList.Last().ad;
                    }

                    mADList.RAdd(
                        new ADCell()
                        {
                            dt = datOf100ms_dt,
                            ad = avgad
                        });
                }
            }
        }

        private void InitList()//初始化列表
        {
            if (mADList == null)
            {
                mADList = new RList<ADCell>(50000);
                mADListToDoNo = mADList.GetLastNo();
                mADListToDoNo_ToFilmInfo = mADListToDoNo;
            }   
            if(mFrameInfoList==null)
		        mFrameInfoList=new RList<FRAME_INFO>(6);
        }


        
        //复位
        void Clear()
        {
            if(mADList!=null)
		        mADList.Clear();
            if (mFrameInfoList != null)
            {
                mFrameInfoList.Clear();
            }
        }

        int Angle2BoltIndex(double angle)
        {
            if(angle<0)
            {
                angle+=360;
            }

            int boltindex = (int)(OrgBoltNo - BoltNo1st +angle*NBolts/360);
            if (boltindex >= NBolts)
            {
                boltindex -= NBolts;
            }
            else if (boltindex < 0)//不应该发生，可能是 OrgBoltNo 出错了。
            {
                boltindex += NBolts;
            }
            return boltindex;
        }
        
        /// <summary>
        /// 返回 -1, 时间点在列表以前;
        /// 返回 0, 时间点在列表中;
        /// 返回 1, 时间点在列表的未来;
        /// </summary>
        /// <param name="dt"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        int ADListFastSearchIndex(DateTime dt, out int index)
        {
            index = 0;

            if (mADList.Count <= 0)
                return 1;


            int offset = (int)((mADList.Last().dt - dt).TotalSeconds * 10);
            index = mADList.Count-offset-1;
            if (index >= mADList.Count)
                return 1;
            else if (index < 0)
                return -1;
            else
            {
                if(mADList[index].dt == dt)
                {
                    return 0;
                }
                else if(mADList[index].dt > dt)
                {
                    do{
                    index--;
                    if(index<0)
                        return -1;
                    }
                    while( mADList[index].dt > dt);
                    return 0;

                } 
                else
                {
                    do{
                    index++;
                    if(index>=mADList.Count)
                        return 1;
                    }
                    while( mADList[index].dt < dt);
                    return index;
                }
            }
        }





        void OnPoll_UpdateADList() 
        {
            
            int adlist_index = mADList.No2Index(mADListToDoNo);
            if (adlist_index >= mADList.Count)
                return;//没有数据需要处理
            if (adlist_index < 0)
                adlist_index = 0;


            for (int i = adlist_index; i < mADList.Count; i++)
            {
                BlowingDetect.FilmInfo filminfo;

                mADListToDoNo = mADList.Index2No(i);
                double filmWidth = (SolveEnable) ? FilmWidth / 1000.0 : 2.3;
                double filmPosH = (SolveEnable) ? FilmPosH / 1000.0 : 0;

                int ret = mPDetect.GetFilmInfo(out filminfo, mADList[i].dt, filmWidth, filmPosH);
                if (ret == 1)//发生在未来
                {
                    return;
                }
                if (filminfo == null)//异常，FilmPosH 错误
                {
                    return;
                }
                if (mADListToDoNo != mADList.Index2No(i))//触发了事件
                { 
                    return;
                }
                ADCell adcell = mADList[i];
                adcell.angle0 = filminfo.angle1;
                adcell.angle1 = filminfo.angle2;
                adcell.direction = filminfo.direction;
                adcell.velocity = filminfo.filmVelocity;
                adcell.rotationCnt = filminfo.rotationCnt;
                adcell.inCV = filminfo.inCV;
            }
            mADListToDoNo = mADList.Index2No(mADList.Count);
        }
        /// <summary>
        /// 状态不一样，人字架已经转了新一圈，需要增加新一幅数据
        /// </summary>
        /// <param name="adcell"></param>
        /// <param name="frameinfo"></param>
        /// <returns></returns>
        bool FrameInfoCheckToNew(ADCell adcell, FRAME_INFO frameinfo) 
        {
            if ((adcell.direction != frameinfo.direction) ||
                (adcell.rotationCnt != frameinfo.rotationCnt) )
                //(Math.Abs(adcell.velocity - frameinfo.velocity) > 2))
            {
                return true;
            }
            else 
            {
                return false;
            }
        }

        
        FR_DATA_TYPE Direction2FR_DATA_TYPE(Misc.DIRECTION direction) 
        {
            if (direction == DIRECTION.FORWARD)
                return FR_DATA_TYPE.INVALID;
            else
                return FR_DATA_TYPE.VALID;
        }




        /// <summary>
        /// 输出的数据都必须是稳定，可靠的。 生成一幅幅测厚仪数据！！！！！！！！！！！！
        /// </summary>
        void OnPoll_UpdateFrameInfo() 
        {
            int adlist_index = mADList.No2Index(mADListToDoNo_ToFilmInfo);//当前在处理的 数据点
            if (adlist_index >= mADList.Count)
                return;//没有数据需要处理
            if (adlist_index < 0)//全部都要处理
                adlist_index = 0;
            //当没有任何一幅数据时， frameinfo_index = mFrameInfoList.Count = 0
            
            int frameinfo_index;
            if(mFrameInfoList.Count>0)
                frameinfo_index = mFrameInfoList.Count - 1;
            else
                frameinfo_index = 0;

            //查找adlist_index 属于哪个frameinfo;
            //当找到，清空里面的.frame,一切重新来！！！
            //当不能找到，说明只是接着上次的处理而已，不需要重新来

            for (int i = 0; i < mFrameInfoList.Count; i++)
            {
                int index_start;
                int index_end;
                FRAME_INFO frameinfo = mFrameInfoList[i];
                if (!GetFilmInfoADIndex(frameinfo, out index_start, out index_end))
                {
                    //数据已经不存在，删除
                    mFrameInfoList.RemoveAt(i);
                    i--;
                    continue;
                }
                if (adlist_index <= index_end)
                {
                    //要处理的数据点，位于 这幅数据 (frameinfo)； 重新计算整幅数据 
                    frameinfo_index = i;
                    //adlist_index = mADList.No2Index(frameinfo.NoStartAD);
                    break;
                }
            }

            int adlist_todo_index = mADList.No2Index(mADListToDoNo);//已经成功获取完位置信息的数据尽头
            for (int i = adlist_index; (i < mADList.Count && i<adlist_todo_index); i++)
            {
                ADCell adcell = mADList[i];
                //mADListToDoNo_ToFilmInfo = mADList.Index2No(i);
                 
                
                FRAME_INFO frameinfo;
                //--------------------------------------------------------------------------------------------------------
                //条件检测
                if (frameinfo_index >= mFrameInfoList.Count)//根本没数据，新建
                {
                    frameinfo = new FRAME_INFO();
                    frameinfo.StartTime = adcell.dt;
                    frameinfo.EndTime = adcell.dt;
                    frameinfo.direction = adcell.direction;
                    frameinfo.velocity = adcell.velocity;
                    frameinfo.rotationCnt = adcell.rotationCnt;
                    frameinfo.NoStartAD = mADList.Index2No(i);
                    frameinfo.NoEndAD = mADList.Index2No(i);
                    frameinfo.SetNBolt(NBolts);
                    mFrameInfoList.RAdd(frameinfo);
                    frameinfo_index = mFrameInfoList.Count - 1;
                }
                else 
                {
                    frameinfo = mFrameInfoList[frameinfo_index];
                    if (i <= mADList.No2Index(frameinfo.NoEndAD)) //清空数据，重新来过, ????????????
                    {
                        frameinfo.StartTime = adcell.dt;
                        frameinfo.EndTime = adcell.dt;
                        frameinfo.direction = adcell.direction;
                        frameinfo.velocity = adcell.velocity;
                        frameinfo.rotationCnt = adcell.rotationCnt;
                        frameinfo.NoStartAD = mADList.Index2No(i);
                        frameinfo.NoEndAD = mADList.Index2No(i);
                        frameinfo.Clear();
                    }
                }
                //--------------------------------------------------------------------------------------------------------


                if (FrameInfoCheckToNew(adcell, frameinfo))
                {
                    //frameinfo.CheckForValid();

                    //一幅数据已经完成
                    if (SolveEnable)
                    {
                        //最后检查一下是否需要解方程
                        frameinfo.SolveFormula();
                    }

                    int[] thicks = frameinfo.frame;
                    if (IsUsedMap)//重新分区
                    {
                        Dictionary<int, int> map = new Dictionary<int, int>();
                        for (int j = 0; j < Map.Count(); j++)
                            map.Add(Map[j].OldNo - 1, Map[j].NewNo - 1);

                        thicks = Misc.MyMath.Map(frameinfo.frame, map);
                    }

                    //推送数据出去
                    DataEvent?.Invoke(this, new RenZiJiaDataEventArgs()
                    {
                        Time = frameinfo.StartTime,
                        EndTime = frameinfo.EndTime,
                        RotationCnt = frameinfo.rotationCnt,
                        Direction = frameinfo.direction,
                        RotatePeriod = mPDetect.RenZiJiaPeriod,
                        Frame = thicks
                    });

                    //更新到下一个frameinfo
                    frameinfo_index++;


                    i--;
                    continue;
                }

                int thick = AD2Thick(adcell.ad);
                if (SolveEnable)//解方程，更新frameinfo.frame
                {
                    int boltindex_0 = Angle2BoltIndex(adcell.angle0);
                    int boltindex_1 = Angle2BoltIndex(adcell.angle1);
                    frameinfo.SetFrameValue(boltindex_0, boltindex_1, thick*2, adcell.dt);
                    
                }
                else//均值，更新frameinfo.frame
                {
                    int boltindex = Angle2BoltIndex(adcell.angle0);
                    frameinfo.SetFrameValue(boltindex, thick, adcell.dt);
                }

                frameinfo.EndTime = adcell.dt;
                frameinfo.NoEndAD = mADList.Index2No(i);
            }
            mADListToDoNo_ToFilmInfo = mADListToDoNo;





            //检测数据更新情况，放入bulkdata
            for (int i = 0; i < mFrameInfoList.Count; i++) 
            {
                FRAME_INFO frameinfo = mFrameInfoList[i];
                if (SolveEnable)
                {
                    //检查是否需要解方程
                      frameinfo.CheckToSolveFormula();
                }

                if (frameinfo.HasChanged)//数据检测过，需要更新bulkdata里面的数据
                {
                    //if (frameinfo.IsValid) //数据有效， 可以有对应的bookmark
                    //{
                    //if (frameinfo.bookmark == -1)
                    //{
                    //    if (free_bookmark != -1)//上一幅数据有问题，接着用上一幅的bookmark
                    //    {
                    //        frameinfo.bookmark = free_bookmark;
                    //        free_bookmark = -1;
                    //    }
                    //}

                    if (frameinfo.bookmark != -1)
                    {
                        if (frameinfo.rotationCnt == 0 && mFrameInfoList.Count > 1)
                        {

                        }
                        
                        int[] thicks = frameinfo.frame;
                        if (IsUsedMap)//重新分区
                        {
                            Dictionary<int, int> map = new Dictionary<int, int>();
                            for (int j = 0; j < Map.Count(); j++)
                                map.Add(Map[j].OldNo - 1, Map[j].NewNo - 1);

                            thicks = Misc.MyMath.Map(frameinfo.frame, map);
                        }

                        if (!mBulkData.Change(frameinfo.bookmark, 0, thicks,
                                Direction2FR_DATA_TYPE(frameinfo.direction),
                                frameinfo.StartTime)) 
                            {
                                //修改失败，肯定是 bulkdata被清空了
                                frameinfo.bookmark = -1;
                            }
                        }

                        if (frameinfo.bookmark == -1)
                        {
                            mBulkData.Save();
                            //添加新的
                            frameinfo.bookmark = mBulkData.Add(frameinfo.frame,
                                Direction2FR_DATA_TYPE(frameinfo.direction),
                                frameinfo.StartTime);
                            if (EPCType == RenZiJiaFixEPCType.EPCA10)
                                mEPCA10.CheckToSample();
                        }
                    //}
                    //else 
                    //{
                    //    if (frameinfo.bookmark != -1) //删除掉整幅数据
                    //    {
                    //        mBulkData.ChangeEmply(frameinfo.bookmark);
                    //        free_bookmark = frameinfo.bookmark;
                    //        frameinfo.bookmark = -1;
                    //    }
                    //}
                    frameinfo.Submit();
                }
            }
            //检查，mFrameInfoList[i].bookmark 应该是递增的

            //for (int i = 0; i < mFrameInfoList.Count-1; i++) 
            //{
            //    if (mFrameInfoList[i].bookmark >= mFrameInfoList[i + 1].bookmark)
            //    { 
            //        //异常
            //        throw new Exception("bookmark  异常");
            //    }
            //}
        }

        protected override void OnPoll()
        {
            if (EPCType == RenZiJiaFixEPCType.EPCA10)
                mEPCA10.OnPoll();

            OnPoll_UpdateADList();
            OnPoll_UpdateFrameInfo();
            mCalFilmLen.OnPoll();
        }

        public override bool Start()
        {
            //以前的 Start()
            IsRunning = true;
            mBulkData.InitPush();

            FObjBase.PollModule.Current.Poll_Config(
                FObjBase.PollModule.POLL_CONFIG.ADD,
                onpoll_func,
                TimeSpan.FromSeconds(1));
            return true;
        }

        bool Load()
        {
            return Misc.SaveToXmlHepler.Load("renzijiafix.xml",this);
        }
        void Save()
        {
            Misc.SaveToXmlHepler.Save("renzijiafix.xml", this);
        }

      
        class CalFilmLen 
        {
            GM_BlowingFix mRenZiJiaFix;
            BlowingDetect mRDetect;
            List<ADCell> mADList=new List<ADCell>();
            int RotationCnt = 3;// 一共要获取3个 这个是最后一个对应的
            DateTime TimeOfMin = DateTime.MinValue;
            DateTime TimeOfMax = DateTime.MinValue;
            int step = 0;
            string State
            {
                set
                {
                    mRenZiJiaFix.CalState = value;
                }
            }
            double MaxR
            {
                set
                {
                    mRenZiJiaFix.MaxR = value;
                }
                get
                {
                    return mRenZiJiaFix.MaxR;
                }
            }
            double MaxRFilmLength
            {
                set
                {
                    mRenZiJiaFix.MaxRFilmLength = value;
                }
                get
                {
                    return mRenZiJiaFix.MaxRFilmLength;
                }
            }
            double MinFilmLength
            {
                get
                {
                    return mRenZiJiaFix.mPDetect.FilmLength - mRenZiJiaFix.FLRange;
                }
            }
            double MaxFilmLength
            {
                get
                {
                    return mRenZiJiaFix.mPDetect.FilmLength + mRenZiJiaFix.FLRange;
                }
            }
            double FilmLength
            {
                get
                {
                    return mRenZiJiaFix.mPDetect.FilmLength;
                }
            }
            public CalFilmLen(GM_BlowingFix renzijiafix) 
            {
                mRenZiJiaFix = renzijiafix;

            }
            public void Start()
            {
                if (step == 0)
                {
                    step = 1;
                    dt_onpoll = DateTime.MinValue;
                    RotationCnt = 3;
                    TimeOfMin = DateTime.MinValue;
                    TimeOfMax = DateTime.MinValue;
                    mADList.Clear();
                    if (mRDetect != null)
                    {
                        mRDetect.Dispose();
                        mRDetect = null;
                    }
                    OnPoll();
                }
            }
            bool Poll()
            {
                switch (step)
                {
                    case 1:
                        {
                            if (Step01())
                            {
                                step = 2;
                                return true;
                            }
                        } break;
                    case 2:
                        {
                            if (Step02())
                            {
                                step = 3;
                                return true;
                            }
                        } break;
                    case 3:
                        {
                            if (Step03())//已经成功获取了时间
                            {
                                step = 4;
                                return true;
                            }
                            else
                            {
                                if (TimeOfMin == DateTime.MinValue)
                                {
                                    step = 2;
                                }
                            }
                        } break;
                    case 4:
                        {
                            Step04();
                            step = 5;
                        } break;
                }
                return false;
            }

            DateTime dt_onpoll = DateTime.MinValue;
            /// <summary>
            /// 每次获取完所有angle，执行,每20秒触发一次
            /// </summary>
            public void OnPoll()
            {
                DateTime now = DateTime.Now;
                if ((dt_onpoll != DateTime.MinValue) && ((now - dt_onpoll) < TimeSpan.FromSeconds(30)))
                    return;

                dt_onpoll = now;
                while (Poll()) ;
            }

            bool Step01() 
            {
                if (mRenZiJiaFix.mPDetect.RotationCnt < 3)
                {
                    RotationCnt = 3;
                    State = "RotationCnt=3 Wait";
                    return true;
                }
                RotationCnt = mRenZiJiaFix.mPDetect.RotationCnt;
                return true;
            }
            /// <summary>
            /// 确保三个rotationcnt 都存在
            /// </summary>
            /// <returns></returns>
            bool Step02()
            {
                if (RotationCnt > mRenZiJiaFix.mPDetect.RotationCnt)
                {
                    State = "RotationCnt=" + 
                        RotationCnt.ToString() + ">" + mRenZiJiaFix.mPDetect.RotationCnt.ToString() + " Wait";
                    
                    return false;
                }

                DateTime min_time;
                DateTime max_time;
                double min_fl = MinFilmLength;
                double max_fl = MaxFilmLength;

                BlowingDetect.GetLimitTimeResult r1 =
                    mRenZiJiaFix.mPDetect.GetLimitTime(
                    RotationCnt-2,
                    min_fl,
                    out min_time);

                BlowingDetect.GetLimitTimeResult r2 =
                    mRenZiJiaFix.mPDetect.GetLimitTime(
                    RotationCnt,
                    max_fl,
                    out max_time);

                string state = "[min_fl=" + min_fl.ToString() + "m (" + min_time.ToLongTimeString() + ") " + r1.ToString() + "]," +
                "[max_fl=" + max_fl.ToString() + "m (" + max_time.ToLongTimeString() + ") " + r2.ToString() + "] ";


                if ((r1 == BlowingDetect.GetLimitTimeResult.NOEXIST) ||
                    (r2 == BlowingDetect.GetLimitTimeResult.NOEXIST))
                {
                    //不可能存在的错误
                    RotationCnt++;
                    state += "Wait 不可能存在的错误 NOEXIST, RotationCnt=" + RotationCnt.ToString();
                    State = state;
                    return false;
                }
                else if ((r1 == BlowingDetect.GetLimitTimeResult.ISFURTURE) ||
                    (r2 == BlowingDetect.GetLimitTimeResult.ISFURTURE))
                {
                    //就是它，等！！！！

                    state += "Wait ,it is the one！！！！";
                    State = state;
                    return false;
                }
                else if ((r1 != BlowingDetect.GetLimitTimeResult.OK) ||
                    (r2 != BlowingDetect.GetLimitTimeResult.OK))
                {
                    //不可能存在的错误
                    RotationCnt++;
                    state += "Wait 不可能存在的错误 ?????, RotationCnt=" + RotationCnt.ToString();
                    State = state;
                    return false;
                }
                TimeOfMin = min_time;
                TimeOfMax = max_time;
                return true;
            }

            bool Step03()
            {
                int copy_index_b=-1;
                int copy_index_e=-1;
                int ret = mRenZiJiaFix.ADListFastSearchIndex(TimeOfMin, out copy_index_b);
                string state;
                state = "TimeOfMin(" + TimeOfMin.ToLongTimeString() + ") (" + copy_index_b.ToString() + ") ";

                if (ret == -1)//发生在过去
                {
                    state += " PAST";
                    State = state;

                    RotationCnt++;
                    TimeOfMin = DateTime.MinValue;
                    TimeOfMax = DateTime.MinValue;
                    return false;
                }
                else if (ret == 1)//发生在未来
                {
                    state += " FUTURE";
                    State = state;

                    return false;
                }

                ret = mRenZiJiaFix.ADListFastSearchIndex(TimeOfMax, out copy_index_e);

                state += " TimeOfMax(" + TimeOfMax.ToLongTimeString() + ") (" + copy_index_e.ToString() +")";


                if (ret == -1)//发生在过去
                {
                    //异常出错
                    throw new ArgumentOutOfRangeException("min_time 在列表中, 但max_time在列表前");
                }
                else if (ret == 1)//发生在未来
                {
                    state += " FUTURE";
                    return false;
                }

                //复制所有scaninfolist
                for (int i = copy_index_b; i <= copy_index_e; i++)
                {
                    mADList.Add(mRenZiJiaFix.mADList[i].Clone());
                }
                //复制 旋转架检测器
                mRDetect = mRenZiJiaFix.mPDetect.Clone();

                //数据足够了
                state += " READY";
                State = state;
                return true;
            }

            void Step04()
            {
                //立刻计算
                Thread t = new Thread(Cal)
                {
                    Priority = ThreadPriority.Lowest,
                    IsBackground = true
                };
                t.Start();
            }
            void Cal()
            {
                //限位时间点，肯定在缓存区的中间！！！！！
                //1.找到限位时间点所在的数据幅
                //2.以该点作为对称点，上下3幅计算thicks,再求相似度
                double min_fl = MinFilmLength;
                double max_fl = MaxFilmLength;
                double maxr = -1;
                double maxr_fl = -1;
                string state;
                double curr_r = CalR(FilmLength);

                for (double i = min_fl; i <= max_fl; i += 0.1)
                {
                    double r = CalR(i);
                    if (r > maxr)
                    {
                        maxr = r;
                        maxr_fl = i;
                    }
                    state = "Cal";
                    state += " (" + ((i - min_fl) / (max_fl - min_fl) * 100).ToString("F0") + "/100)";
                    state += " [" + r.ToString("F3") + "," + i.ToString("N1") + "m]";
                    state += " MaxR=[" + maxr.ToString("F3") + "," + maxr_fl.ToString("N1") + "m]";
                    state += " CurrR=[" + curr_r.ToString("F3") + "," + FilmLength.ToString("N1") + "m]";
                    State = state;
                }

                MaxR = maxr;
                MaxRFilmLength = maxr_fl;
                State = "Cal OK";
                step = 0;
            }
            double CalR(double filmLength)
            {
                mRDetect.FilmLength = filmLength;
                
                for (int i = 0; i < mADList.Count(); i++) 
                {
                    ADCell c = mADList[i];
                    BlowingDetect.FilmInfo filminfo;
                    int ret = mRDetect.GetFilmInfo(out filminfo, mADList[i].dt, 2.3, 0);
                    if (ret != 0) 
                    {
                        //异常
                        return 0;
                    }
                    c.direction = filminfo.direction;
                    c.angle0 = filminfo.angle1;
                    c.angle1 = filminfo.angle2;
                    c.rotationCnt = filminfo.rotationCnt;
                    c.velocity = filminfo.filmVelocity;
                    c.inCV = filminfo.inCV;
                }

                //找到 rotationCnt 为 RotationCnt-2， 与 RotationCnt-1 的数据，进行相似度计算
                //直接使用角度划分数据， 360° 
                //360个数据太多，使用120个数据
                
                int[] thicks1 = new int[120];
                int[] thicks2 = new int[120];
                for (int i = 0; i < 120; i++)
                {
                    thicks1[i] = Misc.MyBase.NULL_VALUE;
                    thicks2[i] = Misc.MyBase.NULL_VALUE;
                }

                int sum = 0;
                int cnt = 0;
                //int angle = -1;
                int angle_3 = -1;
                int rotationcnt=0;
                for (int i = 0; i < mADList.Count(); i++) 
                {
                    ADCell c = mADList[i]; 
                    if(cnt==0)
                    {
                        if (Misc.MyBase.ISVALIDATA(c.ad))
                        {
                            sum += c.ad;
                            cnt++;
                        }

                        angle_3 = (int)(c.angle0 / 3);
                        rotationcnt = c.rotationCnt;
                    }
                    else{
                        bool change=false;
                        if ((rotationcnt != c.rotationCnt) ||
                            (angle_3 != (int)(c.angle0 / 3)))
                        {
                            change = true;
                        }
                        if (!change)
                        {
                            if (Misc.MyBase.ISVALIDATA(c.ad))
                            {
                                sum += c.ad;
                                cnt++;
                            }
                        }
                        else
                        {
                            int[] thicks=null;
                            if (rotationcnt == RotationCnt - 2)
                                thicks = thicks1;
                            else if (rotationcnt == RotationCnt - 1)
                                thicks = thicks2;
                            if (thicks != null) 
                            {
                                int index = angle_3;
                                if (index >= 120)
                                    index = 119;
                                else if (index < 0)
                                    index = 0;
                                if (cnt > 0)
                                    thicks[index] = sum / cnt;
                                else
                                    thicks[index] = Misc.MyBase.NULL_VALUE;
                            }
                            sum = 0;
                            cnt = 0;
                            if (Misc.MyBase.ISVALIDATA(c.ad))
                            {
                                sum += c.ad;
                                cnt++;
                            }
                            rotationcnt = c.rotationCnt;
                            angle_3 = (int)(c.angle0 / 3);
                        }
                    }
                }


                //计算相似性
                return Misc.MyMath.Correl(thicks1,thicks2);
            }
        }
    }






}
