﻿using FLY.OBJComponents.Client;
using FLY.Weight.Client;
using FLY.Weight.Common;
using LiveCharts;
using LiveCharts.Configurations;
using LiveCharts.Wpf;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;

using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace FLY.Weight.UI.Client.UIModule
{
    /// <summary>
    /// FlowGraph.xaml 的交互逻辑
    /// </summary>
    public partial class FlowGraph : UserControl
    {
        FlowGraphModelView flowGraphModelView;
        public FlowGraph()
        {
            InitializeComponent();
        }
        public void Init(int id)
        {
            flowGraphModelView = new FlowGraphModelView();
            flowGraphModelView.InitData();
            this.DataContext = flowGraphModelView;
            stackpanel_window.DataContext = flowGraphModelView.bufferWindow;
        }
        public void Debug()
        {
            flowGraphModelView = new FlowGraphModelView();
            flowGraphModelView.InitData_Debug();
            this.DataContext = flowGraphModelView;
            //TODO 有问题
            //itemsControl.ItemsSource = flowGraphModelView.Series;
            //stackpanel_window.DataContext = flowGraphModelView.bufferWindow;
        }
        //private void ListBox_OnPreviewMouseDown(object sender, MouseButtonEventArgs e)
        //{
        //    var item = ItemsControl.ContainerFromElement(ListBox, (DependencyObject)e.OriginalSource) as ListBoxItem;
        //    if (item == null) return;

        //    var series = (StackedAreaSeries)item.Content;
        //    if (series.Visibility == Visibility.Visible)
        //        series.Visibility = Visibility.Hidden;
        //    else
        //        series.Visibility = Visibility;
        //}

        private void button_info_Click(object sender, RoutedEventArgs e)
        {
            Window_GraphSet w = new Window_GraphSet();
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);
            w.ShowDialog();
        }

        private void button_newest_Click(object sender, RoutedEventArgs e)
        {
            flowGraphModelView.Newest();
        }

        private void button_pre_Click(object sender, RoutedEventArgs e)
        {
            flowGraphModelView.Pre();
        }

        private void button_next_Click(object sender, RoutedEventArgs e)
        {
            flowGraphModelView.Next();
        }

        private void button_abc_Click(object sender, RoutedEventArgs e)
        {
            StackedAreaSeries series = ((Button)sender).Tag as StackedAreaSeries;
            if (series == null) return;
            
            if (series.Visibility == Visibility.Visible)
                series.Visibility = Visibility.Hidden;
            else
                series.Visibility = Visibility;
        }

        private void button_table_Click(object sender, RoutedEventArgs e)
        {
            NavigationService ns = Application.Current.Properties["NavigationService"] as NavigationService;
            if (ns != null)
            {
                Page_FlowTable p = new Page_FlowTable();
                p.Init();
                ns.Navigate(p);
            }
        }

    }
    public class FlowGraphModelView : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        WeightSystemClient weightSystemClient;
        public BufferWindow<FlyData_Flow> bufferWindow;
        public FlowGraphParams graphParams;

        const int ValuesCap = 200;
        #region property
        /// <summary>
        /// 曲线, 是由 bufferWindow.Record 生成的
        /// Values 最大数量只有 ValuesCap
        /// 平均从 bufferWindow.Record提取， 也就是Record有1000个数，也只会间隔抽取200个画出来
        /// </summary>
        public ChartValues<FlyData_Flow> Values { get; set; }

        /// <summary>
        /// 折径曲线 XY 定义
        /// </summary>
        //public CartesianMapper<FlyData_Flow> Mapper_ { get; set; }


        /// <summary>
        /// X轴格式
        /// </summary>
        public Func<double, string> DateTimeFormatter { get; set; }

        /// <summary>
        /// y轴格式
        /// </summary>
        public Func<double, string> YFormatter { get; set; }

        /// <summary>
        /// 折径目标值
        /// </summary>
        public double AxisYTarget { get; set; }

        /// <summary>
        /// 折径 最大值
        /// </summary>
        public double MaxValue { get; set; }
        /// <summary>
        /// 折径 最小值
        /// </summary>
        public double MinValue { get; set; }

        /// <summary>
        /// 开始时间
        /// </summary>
        public DateTime BeginTime { get; set; }

        /// <summary>
        /// 结束数据
        /// </summary>
        public DateTime EndTime { get; set; }

        /// <summary>
        /// 曲线
        /// </summary>
        public SeriesCollection Series { get; set; } = new SeriesCollection();
        #endregion

        public FlowGraphModelView()
        {
            Init();
        }



        void Init()
        {
            //Mapper_FilmWidth = Mappers.Xy<FlyData_IBCCtrl>()
            //    .X(value => value.Time.Ticks)
            //    .Y(value => value.FilmWidth);

            DateTimeFormatter = value => new DateTime((long)value).ToString("HH:mm:ss");

            YFormatter = value => value.ToString("F1");

            Values = new ChartValues<FlyData_Flow>();
        }

        public void InitData()
        {
            graphParams = SysParam.Current.GraphParams;
            weightSystemClient = TDGage.Current.mWeighterCsService;

            bufferWindow = new BufferWindow<FlyData_Flow>(weightSystemClient.FlowList, graphParams.WindowSize);

            Misc.BindingOperations.SetBinding(graphParams, "WindowSize", bufferWindow, "Size");

            Misc.BindingOperations.SetBinding(
                weightSystemClient.Accessory, "TotalFlowSetting", this, "AxisYTarget");
            
            ToValues();
            bufferWindow.Record.CollectionChanged += (object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e) =>
            {
                ToValues();
            };

            //根据 weightSystemClient.Item创建Series
            BindingSeries();
            weightSystemClient.ResetItemsEvent += BindingSeries;
            


            Update_y();


            this.PropertyChanged += (s, e) =>
            {
                if (e.PropertyName == "AxisYTarget")
                    Update_y();
            };

            graphParams.PropertyChanged += (s, e) => {
                if (e.PropertyName == "IsAxisYRangeAuto")
                {
                    Update_y();
                }
                else if (e.PropertyName == "AxisYRange")
                {
                    Update_y();
                }
            };

            
        }

        class GetMapperOfFlyData_Flow
        {
            int Index = 0;
            public GetMapperOfFlyData_Flow(int index)
            {
                Index = index;
            }
            public CartesianMapper<FlyData_Flow> GetCartesianMapper()
            {
                return Mappers.Xy<FlyData_Flow>()
                    .X(value => value.Time.Ticks)
                    .Y(value =>
                    {
                        if (Index < value.Items.Count())
                            return value.Items[Index].Flow;
                        else
                            return double.NaN;
                    });
            }

        }
        void BindingSeries()
        {
            if (Series.Count < weightSystemClient.Items.Count)
            {
                for (int i = Series.Count; i < weightSystemClient.Items.Count; i++)
                {
                    StackedAreaSeries series = new StackedAreaSeries();
                    series.Title = weightSystemClient.Items[i].Number;
                    series.Fill = weightSystemClient.Items[i].Color;
                    series.SetBinding(StackedAreaSeries.ValuesProperty, new Binding("Values") { Source = this });
                    series.Configuration = new GetMapperOfFlyData_Flow(i).GetCartesianMapper();
                    Series.Add(series);
                }
            }
            else if (Series.Count > weightSystemClient.Items.Count)
            {
                //删除
                while (Series.Count > weightSystemClient.Items.Count)
                {
                    Series.RemoveAt(Series.Count - 1);
                }
            }
        }
        void ToValues()
        {
            if (bufferWindow.Record.Count <= ValuesCap)
            {
                Values.Clear();
                Values.AddRange(bufferWindow.Record);
            }
            else
            {
                Values.Clear();
                for (int i = 0; i < ValuesCap; i++)
                {
                    int idx = i * bufferWindow.Record.Count / ValuesCap;
                    Values.Add(bufferWindow.Record[idx]);
                }
            }
            if (Values.Count() > 0)
            {
                BeginTime = Values.First().Time;
                EndTime = Values.Last().Time;
            }
            else
            {
                BeginTime = DateTime.MinValue;
                EndTime = DateTime.MinValue;
            }
        }

        void Update_y()
        {
            if (graphParams.IsAxisYRangeAuto)
            {
                MaxValue = double.NaN;
                MinValue = double.NaN;
            }
            else
            {
                MaxValue = AxisYTarget + graphParams.AxisYRange;
                MinValue = AxisYTarget - graphParams.AxisYRange;
            }
        }

        public void InitData_Debug()
        {
            
            graphParams = SysParam.Current.GraphParams;
            DateTime date = DateTime.Now;

            Random r = new Random();
            double noise = 2;

            double[] flows = new double[] { 50, 300, 300, 280, 20 };

            ObservableCollection<FlyData_Flow> Record = new ObservableCollection<FlyData_Flow>();
            for (int i = 0; i < 1000; i++)
            {
                FlyData_Flow flyData_Flow = new FlyData_Flow();
                flyData_Flow.Time = date.AddSeconds(1 * i);
                flyData_Flow.Items = new FlyData_FlowItem[flows.Count()];
                for (int j=0;j< flows.Count(); j++)
                {
                    flyData_Flow.Items[j] = new FlyData_FlowItem()
                    {
                        Flow = (float)(flows[j] + r.NextDouble() * noise - noise / 2)
                    };
                }
                Record.Add(flyData_Flow);
            }

            if (Record.Count <= ValuesCap)
            {
                Values.Clear();
                Values.AddRange(Record);
            }
            else
            {
                Values.Clear();
                for (int i = 0; i < ValuesCap; i++)
                {
                    int idx = i * Record.Count / ValuesCap;
                    Values.Add(Record[idx]);
                }
            }

            if (Values.Count() > 0)
            {
                BeginTime = Values.First().Time;
                EndTime = Values.Last().Time;
            }
            else
            {
                BeginTime = DateTime.MinValue;
                EndTime = DateTime.MinValue;
            }

            AxisYTarget = flows.Sum();


            for (int i = 0; i < flows.Count(); i++)
            {
                StackedAreaSeries series = new StackedAreaSeries();
                series.Title = ((char)('A' + i)).ToString();
                series.Fill = WeighterColorDB.GetSelf(series.Title);
                series.SetBinding(StackedAreaSeries.ValuesProperty, new Binding("Values") { Source = this });
                series.Configuration = new GetMapperOfFlyData_Flow(i).GetCartesianMapper();
                Series.Add(series);
            }
            graphParams.IsAxisYRangeAuto = true;
            if (graphParams.IsAxisYRangeAuto)
            {
                MaxValue = double.NaN;
                MinValue = double.NaN;
            }
            else
            {
                MaxValue = AxisYTarget + graphParams.AxisYRange;
                MinValue = AxisYTarget - graphParams.AxisYRange;
            }
        }

        public void Newest()
        {
            bufferWindow.MoveNewest();
        }
        public void Next()
        {
            bufferWindow.MoveNextPage();
        }
        public void Pre()
        {
            bufferWindow.MovePrePage();
        }
    }

    public class FlowGraphParams : INotifyPropertyChanged, Misc.ISaveToXml
    {
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// 窗口显示数据总量
        /// </summary>
        public int WindowSize { get; set; } = 1000;

        /// <summary>
        /// 折径，进风-出风, Y轴 自动设置 MaxValue， MinValue, 
        /// </summary>
        public bool IsAxisYRangeAuto { get; set; }=true;
        /// <summary>
        /// 折径，Y轴显示范围， 中间值一定是 FilmWidthSet, 单位mm
        /// MaxValue = FilmWidthSet+FilmWidthAxisYRange
        /// MinValue = FilmWidthSet-FilmWidthAxisYRange
        /// </summary>
        public int AxisYRange { get; set; } = 20;

        public string[] GetSavePropertyNames()
        {
            return new string[]{
                "WindowSize",
                "AxisYRange",
                "IsAxisYRangeAuto"
            };
        }
    }

    public class UIModule_FlowGraph : FLY.UI.Module.IUIModule
    {
        /// <summary>
        /// 控件标题
        /// 它的值取决于culture
        /// </summary>
        public string Title
        {
            get
            {
                return "称重流量";
            }
        }

        /// <summary>
        /// 控件
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetComponent(int id)
        {
            FlowGraph graph = new FlowGraph();
            graph.Init(id);
            return graph;
        }

        /// <summary>
        /// 控件缩略图，用于编辑界面时，大致看看
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetThumbnail(int id)
        {
            return new System.Windows.Controls.Grid();
        }


        /// <summary>
        /// 给出全部控件ID, 控件自行删除没有的参数
        /// </summary>
        /// <param name="IDs"></param>
        public void MatchParam(int[] IDs)
        {
        }
    }
}
