﻿using FLY.Thick.RemoteHistory;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace FLY.Weight.Common
{
    public class FlyData_FlowItem
    {
        /// <summary>
        /// 流量
        /// </summary>
        public float Flow { get; set; }
        
        /// <summary>
        /// 比例
        /// </summary>
        public float ScrewPDisp { get; set; }
        
        /// <summary>
        /// 频率
        /// </summary>
        public float ScrewMotorFreq { get; set; }

    }
    public class FlyData_Flow : IFlyData
    {
        /// <summary>
        /// 时间点
        /// </summary>
        public DateTime Time
        {
            get;
            set;
        }
        /// <summary>
        /// 总流量
        /// </summary>
        public float Total
        {
            get;set;
        }
        /// <summary>
        /// 各上料流量
        /// </summary>
        public FlyData_FlowItem[] Items { get; set; }

        public string GetHeader()
        {
            string header = "时间";
            header+=",总流量";

            for (int i = 0; i < Items.Count(); i++)
            {
                string name = ""+(char)('A' + i);
                header += "," + name + "层流量";
                header += "," + name + "层比例";
                header += "," + name + "层频率";
            }

            return header;
        }
        public override string ToString()
        {
            string str;
            str = Time.ToString();//时间
            str += "," + Total.ToString("F1");//总流量
            for (int i = 0; i < Items.Count(); i++)
            {
                str += "," + Items[i].Flow.ToString("F1");//单层流量
                str += "," + Items[i].ScrewPDisp.ToString("F1");//单层比例
                str += "," + Items[i].ScrewMotorFreq.ToString("F1");//单层频率
            }
            return str;
        }
        public bool TryParse(string header_str, string str)
        {
            string[] items = header_str.Split(new char[] { ',' });

            if (items.Length < (2+3*3))
                return false;
            int items_cnt = (items.Count() - 2) / 3;

            items = str.Split(new char[] { ',' });
            int idx = 0;
            DateTime t;
            float f;

            //时间
            if (!DateTime.TryParse(items[idx], out t))
                return false;
            Time = t;
            idx++;
            
            //总流量
            if (!float.TryParse(items[idx], out f))
                return false;
            Total = f;
            idx++;

            Items = new FlyData_FlowItem[items_cnt];

            for (int i = 0; i < items_cnt; i++)
            {
                Items[i] = new FlyData_FlowItem();

                if (!float.TryParse(items[idx], out f))//单层流量
                    return false;
                Items[i].Flow = f;
                idx++;

                if (!float.TryParse(items[idx], out f))//单层比例
                    return false;
                Items[i].ScrewPDisp = f;
                idx++;

                if (!float.TryParse(items[idx], out f))//单层频率
                    return false;
                Items[i].ScrewMotorFreq = f;
                idx++;
            }
            return true;
        }

    }
   
}
