﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Reflection;
using System.Diagnostics;
using System.Windows.Media;


namespace FLY.Weight.Common
{
    public class WeighterC:INotifyPropertyChanged
    {
        public class DataValue
        {
            public object readVal;
            public object writeVal;
            /// <summary>
            /// 通过set设定过writeVal
            /// </summary>
            public bool setVal;
            public Type type;
            public DataValue(object val1, object val2, Type type)
            {
                readVal = val1 == null ? val2 : val1;
                writeVal = val2 == null ? val1 : val2;
                this.type = type;
            }
        }
        /// <summary>
        /// datas内容有WeighterC的属性确定，
        /// </summary>
        public Dictionary<string, DataValue> datas = new Dictionary<string, DataValue>();

        public WeighterC(string nam, int binCnt)
        {
            Name = nam;
            Type type = this.GetType();
            PropertyInfo[] properties = type.GetProperties();
            foreach (PropertyInfo p in properties)
            {
                datas.Add(p.Name, new DataValue(null, null, p.PropertyType));
            }

            BinCnt = binCnt;
            BinsBinding();
        }


        #region INotifyPropertyChanged 成员

        public event PropertyChangedEventHandler PropertyChanged;
        public void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        #endregion

        #region IWeighterC实现
        public string Name
        {
            get;
            set;
        }
        /// <summary>
        /// 名称:A,B,C,D,E
        /// </summary>
        public string Number { get { return Name; } }
        /// <summary>
        /// 颜色
        /// </summary>
        public Brush Color { get; set; }
        /// <summary>
        /// 料仓数
        /// </summary>
        public int BinCnt { get; protected set; }
        /// <summary>
        /// 主动更新所有参数（属性）的值
        /// </summary>
        public void UpdateCDataAll()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// 获取所有属性名。
        /// </summary>
        /// <returns></returns>
        public List<string> GetAllPropertyName()
        {
            List<string> names = new List<string>();
            foreach (string key in datas.Keys)
            {
                names.Add(key);
            }
            return names;
        }

        /// <summary>
        /// 设定参数（属性）的值，触发属性值变化“通知”,writeVal
        /// </summary>
        /// <param name="pname">属性名</param>
        /// <param name="val">属性值</param>
        public void SetData(string pname, object v)
        {
            if (datas.ContainsKey(pname))
            {
                if (((v == null) && (datas[pname].readVal != null)) ||
                    (!v.Equals(datas[pname].readVal)))
                {
                    datas[pname].readVal = v;
                    NotifyPropertyChanged(pname);
                }
            }
        }

        /// <summary>
        /// 设定参数（属性）的值，触发属性值变化“通知”,writeVal
        /// </summary>
        /// <param name="pname">属性名</param>
        /// <param name="val">属性值</param>
        public void SetData(string pname, string val)
        {
            if (datas.ContainsKey(pname))
            {
				object v = Misc.Converter.Parse(val, datas[pname].type);
                SetData(pname, v);
            }
        }
        


        /// <summary>
        /// 获取设定的属性值，writeVal
        /// </summary>
        /// <param name="pname"></param>
        /// <returns></returns>
        public object GetData(string pname)
        {
            if (datas.ContainsKey(pname))
			{
				return datas[pname].readVal;
			}
            else 
                return null;
        }


        #region Weighter的功能属性

        /// <summary>
        /// 在get或set内部调用获取当前属性名字的string
        /// </summary>
        /// <returns></returns>
        static string GetMethodName(int stack)
        {
            var method = new StackFrame(stack).GetMethod(); // 这里忽略1层堆栈，也就忽略了当前方法GetMethodName，这样拿到的就正好是外部调用GetMethodName的方法信息
            var property = (
                      from p in method.DeclaringType.GetProperties(
                               BindingFlags.Instance |
                               BindingFlags.Static |
                               BindingFlags.Public |
                               BindingFlags.NonPublic)
                      where p.GetGetMethod(true) == method || p.GetSetMethod(true) == method
                      select p).FirstOrDefault();
            return property == null ? method.Name : property.Name;
        }

        /// <summary>
        /// 设置属性set值，触发OBJ发包改写
        /// </summary>
        /// <param name="value"></param>
        void SetProperValue(object value)
        {
            string pname = GetMethodName(2);
            if (datas.ContainsKey(pname))
            {
                DataValue dv = datas[pname];

				if(dv.readVal != value)
				{
					dv.readVal = value;
					NotifyPropertyChanged(pname);
				}
            }
        }




        #endregion


        #endregion


        #region 辅助代码生成
        /// <summary>
        /// 报警开关
        /// </summary>
        public bool AlarmIsOn
        {
            get
            {
                if (datas.ContainsKey("AlarmIsOn"))
                {
                    DataValue dv = datas["AlarmIsOn"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 报警复位
        /// </summary>
        public bool IsAlarmReseted
        {
            get
            {
                if (datas.ContainsKey("IsAlarmReseted"))
                {
                    DataValue dv = datas["IsAlarmReseted"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 总流量设定
        /// </summary>
        public float TotalFlowSetting
        {
            get
            {
                if (datas.ContainsKey("TotalFlowSetting"))
                {
                    DataValue dv = datas["TotalFlowSetting"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 总流量显示
        /// </summary>
        public float TotalFlow
        {
            get
            {
                if (datas.ContainsKey("TotalFlow"))
                {
                    DataValue dv = datas["TotalFlow"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 总产量
        /// </summary>
        public float TotalProduction
        {
            get
            {
                if (datas.ContainsKey("TotalProduction"))
                {
                    DataValue dv = datas["TotalProduction"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 码轮周长mm
        /// </summary>
        public float WheelPerimeter
        {
            get
            {
                if (datas.ContainsKey("WheelPerimeter"))
                {
                    DataValue dv = datas["WheelPerimeter"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 码轮脉冲
        /// </summary>
        public float WheelPulse
        {
            get
            {
                if (datas.ContainsKey("WheelPulse"))
                {
                    DataValue dv = datas["WheelPulse"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 平均密度
        /// </summary>
        public float Density
        {
            get
            {
                if (datas.ContainsKey("Density"))
                {
                    DataValue dv = datas["Density"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 边料不回收
        /// </summary>
        public bool IsRimNoRecycle
        {
            get
            {
                if (datas.ContainsKey("IsRimNoRecycle"))
                {
                    DataValue dv = datas["IsRimNoRecycle"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 边料kg/h
        /// </summary>
        public float RimCharge
        {
            get
            {
                if (datas.ContainsKey("RimCharge"))
                {
                    DataValue dv = datas["RimCharge"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 厚度um
        /// </summary>
        public float Thickness
        {
            get
            {
                if (datas.ContainsKey("Thickness"))
                {
                    DataValue dv = datas["Thickness"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 当前线速度m/min
        /// </summary>
        public float CurrentVelocity
        {
            get
            {
                if (datas.ContainsKey("CurrentVelocity"))
                {
                    DataValue dv = datas["CurrentVelocity"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 当前线速度设置m/min
        /// </summary>
        public float CurrentVelocitySet
        {
            get
            {
                if (datas.ContainsKey("CurrentVelocitySet"))
                {
                    DataValue dv = datas["CurrentVelocitySet"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }
         

        /// <summary>
        /// 吹膜宽度mm
        /// </summary>
        public float TotalFilmWidth
        {
            get
            {
                if (datas.ContainsKey("TotalFilmWidth"))
                {
                    DataValue dv = datas["TotalFilmWidth"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 切边mm
        /// </summary>
        public float RimWidth
        {
            get
            {
                if (datas.ContainsKey("RimWidth"))
                {
                    DataValue dv = datas["RimWidth"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 收卷宽度mm
        /// </summary>
        public float ActFilmWidth
        {
            get
            {
                if (datas.ContainsKey("ActFilmWidth"))
                {
                    DataValue dv = datas["ActFilmWidth"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 设定厚度um
        /// </summary>
        public float SetThickness
        {
            get
            {
                if (datas.ContainsKey("SetThickness"))
                {
                    DataValue dv = datas["SetThickness"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 理论速度m/min
        /// </summary>
        public float TargetVelocity
        {
            get
            {
                if (datas.ContainsKey("TargetVelocity"))
                {
                    DataValue dv = datas["TargetVelocity"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 内收卷当前m
        /// </summary>
        public float ACurrentLen
        {
            get
            {
                if (datas.ContainsKey("ACurrentLen"))
                {
                    DataValue dv = datas["ACurrentLen"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 内收卷当前kg
        /// </summary>
        public float ACurrent
        {
            get
            {
                if (datas.ContainsKey("ACurrent"))
                {
                    DataValue dv = datas["ACurrent"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 内收卷上卷kg
        /// </summary>
        public float ALast
        {
            get
            {
                if (datas.ContainsKey("ALast"))
                {
                    DataValue dv = datas["ALast"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 外收卷当前m
        /// </summary>
        public float BCurrentLen
        {
            get
            {
                if (datas.ContainsKey("BCurrentLen"))
                {
                    DataValue dv = datas["BCurrentLen"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 外收卷当前kg
        /// </summary>
        public float BCurrent
        {
            get
            {
                if (datas.ContainsKey("BCurrent"))
                {
                    DataValue dv = datas["BCurrent"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 外收卷上卷kg
        /// </summary>
        public float BLast
        {
            get
            {
                if (datas.ContainsKey("BLast"))
                {
                    DataValue dv = datas["BLast"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 内收卷清零
        /// </summary>
        public bool AClear
        {
            get
            {
                if (datas.ContainsKey("AClear"))
                {
                    DataValue dv = datas["AClear"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 外收卷清零
        /// </summary>
        public bool BClear
        {
            get
            {
                if (datas.ContainsKey("BClear"))
                {
                    DataValue dv = datas["BClear"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }


        /// <summary>
        /// 累计产量
        /// </summary>
        public float CumulativeProduction
        {
            get
            {
                if (datas.ContainsKey("CumulativeProduction"))
                {
                    DataValue dv = datas["CumulativeProduction"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 配料斗重量显示
        /// </summary>
        public float MixBucketWeight
        {
            get
            {
                if (datas.ContainsKey("MixBucketWeight"))
                {
                    DataValue dv = datas["MixBucketWeight"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 配料斗阀门
        /// </summary>
        public bool BucketValveIsOpen
        {
            get
            {
                if (datas.ContainsKey("BucketValveIsOpen"))
                {
                    DataValue dv = datas["BucketValveIsOpen"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 配料操作
        /// </summary>
        public bool MixIsOn
        {
            get
            {
                if (datas.ContainsKey("MixIsOn"))
                {
                    DataValue dv = datas["MixIsOn"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 失重仓重量
        /// </summary>
        public float BinWeight
        {
            get
            {
                if (datas.ContainsKey("BinWeight"))
                {
                    DataValue dv = datas["BinWeight"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 当前产量
        /// </summary>
        public float CurrentFlow
        {
            get
            {
                if (datas.ContainsKey("CurrentFlow"))
                {
                    DataValue dv = datas["CurrentFlow"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 设定产量
        /// </summary>
        public float FlowSetting
        {
            get
            {
                if (datas.ContainsKey("FlowSetting"))
                {
                    DataValue dv = datas["FlowSetting"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 运行比例
        /// </summary>
        public float ScrewPDisp
        {
            get
            {
                if (datas.ContainsKey("ScrewPDisp"))
                {
                    DataValue dv = datas["ScrewPDisp"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 设定比例
        /// </summary>
        public float ScrewPSet
        {
            get
            {
                if (datas.ContainsKey("ScrewPSet"))
                {
                    DataValue dv = datas["ScrewPSet"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 运行频率
        /// </summary>
        public float ScrewMotorFreq
        {
            get
            {
                if (datas.ContainsKey("ScrewMotorFreq"))
                {
                    DataValue dv = datas["ScrewMotorFreq"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 设定频率
        /// </summary>
        public float ScrewManualFreq
        {
            get
            {
                if (datas.ContainsKey("ScrewManualFreq"))
                {
                    DataValue dv = datas["ScrewManualFreq"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 螺杆模式
        /// </summary>
        public bool ScrewIsAutoMode
        {
            get
            {
                if (datas.ContainsKey("ScrewIsAutoMode"))
                {
                    DataValue dv = datas["ScrewIsAutoMode"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 螺杆操作
        /// </summary>
        public bool ScrewMotorIsOn
        {
            get
            {
                if (datas.ContainsKey("ScrewMotorIsOn"))
                {
                    DataValue dv = datas["ScrewMotorIsOn"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1#比例设定
        /// </summary>
        public float MixPSet_1
        {
            get
            {
                if (datas.ContainsKey("MixPSet_1"))
                {
                    DataValue dv = datas["MixPSet_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2#比例设定
        /// </summary>
        public float MixPSet_2
        {
            get
            {
                if (datas.ContainsKey("MixPSet_2"))
                {
                    DataValue dv = datas["MixPSet_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3#比例设定
        /// </summary>
        public float MixPSet_3
        {
            get
            {
                if (datas.ContainsKey("MixPSet_3"))
                {
                    DataValue dv = datas["MixPSet_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4#比例设定
        /// </summary>
        public float MixPSet_4
        {
            get
            {
                if (datas.ContainsKey("MixPSet_4"))
                {
                    DataValue dv = datas["MixPSet_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5#比例设定
        /// </summary>
        public float MixPSet_5
        {
            get
            {
                if (datas.ContainsKey("MixPSet_5"))
                {
                    DataValue dv = datas["MixPSet_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1#比例显示
        /// </summary>
        public float MixPDisp_1
        {
            get
            {
                if (datas.ContainsKey("MixPDisp_1"))
                {
                    DataValue dv = datas["MixPDisp_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2#比例显示
        /// </summary>
        public float MixPDisp_2
        {
            get
            {
                if (datas.ContainsKey("MixPDisp_2"))
                {
                    DataValue dv = datas["MixPDisp_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3#比例显示
        /// </summary>
        public float MixPDisp_3
        {
            get
            {
                if (datas.ContainsKey("MixPDisp_3"))
                {
                    DataValue dv = datas["MixPDisp_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4#比例显示
        /// </summary>
        public float MixPDisp_4
        {
            get
            {
                if (datas.ContainsKey("MixPDisp_4"))
                {
                    DataValue dv = datas["MixPDisp_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5#比例显示
        /// </summary>
        public float MixPDisp_5
        {
            get
            {
                if (datas.ContainsKey("MixPDisp_5"))
                {
                    DataValue dv = datas["MixPDisp_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1#重量设定
        /// </summary>
        public float MixSet_1
        {
            get
            {
                if (datas.ContainsKey("MixSet_1"))
                {
                    DataValue dv = datas["MixSet_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2#重量设定
        /// </summary>
        public float MixSet_2
        {
            get
            {
                if (datas.ContainsKey("MixSet_2"))
                {
                    DataValue dv = datas["MixSet_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3#重量设定
        /// </summary>
        public float MixSet_3
        {
            get
            {
                if (datas.ContainsKey("MixSet_3"))
                {
                    DataValue dv = datas["MixSet_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4#重量设定
        /// </summary>
        public float MixSet_4
        {
            get
            {
                if (datas.ContainsKey("MixSet_4"))
                {
                    DataValue dv = datas["MixSet_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5#重量设定
        /// </summary>
        public float MixSet_5
        {
            get
            {
                if (datas.ContainsKey("MixSet_5"))
                {
                    DataValue dv = datas["MixSet_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1#重量显示
        /// </summary>
        public float MixDisp_1
        {
            get
            {
                if (datas.ContainsKey("MixDisp_1"))
                {
                    DataValue dv = datas["MixDisp_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2#重量显示
        /// </summary>
        public float MixDisp_2
        {
            get
            {
                if (datas.ContainsKey("MixDisp_2"))
                {
                    DataValue dv = datas["MixDisp_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3#重量显示
        /// </summary>
        public float MixDisp_3
        {
            get
            {
                if (datas.ContainsKey("MixDisp_3"))
                {
                    DataValue dv = datas["MixDisp_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4#重量显示
        /// </summary>
        public float MixDisp_4
        {
            get
            {
                if (datas.ContainsKey("MixDisp_4"))
                {
                    DataValue dv = datas["MixDisp_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5#重量显示
        /// </summary>
        public float MixDisp_5
        {
            get
            {
                if (datas.ContainsKey("MixDisp_5"))
                {
                    DataValue dv = datas["MixDisp_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1#放料
        /// </summary>
        public bool MixLight_1
        {
            get
            {
                if (datas.ContainsKey("MixLight_1"))
                {
                    DataValue dv = datas["MixLight_1"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2#放料
        /// </summary>
        public bool MixLight_2
        {
            get
            {
                if (datas.ContainsKey("MixLight_2"))
                {
                    DataValue dv = datas["MixLight_2"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3#放料
        /// </summary>
        public bool MixLight_3
        {
            get
            {
                if (datas.ContainsKey("MixLight_3"))
                {
                    DataValue dv = datas["MixLight_3"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4#放料
        /// </summary>
        public bool MixLight_4
        {
            get
            {
                if (datas.ContainsKey("MixLight_4"))
                {
                    DataValue dv = datas["MixLight_4"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5#放料
        /// </summary>
        public bool MixLight_5
        {
            get
            {
                if (datas.ContainsKey("MixLight_5"))
                {
                    DataValue dv = datas["MixLight_5"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 配料斗卸载
        /// </summary>
        public bool MixUnload
        {
            get
            {
                if (datas.ContainsKey("MixUnload"))
                {
                    DataValue dv = datas["MixUnload"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 配料斗重量设置
        /// </summary>
        public float MixSet
        {
            get
            {
                if (datas.ContainsKey("MixSet"))
                {
                    DataValue dv = datas["MixSet"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 配料斗重量显示
        /// </summary>
        public float MixDisp
        {
            get
            {
                if (datas.ContainsKey("MixDisp"))
                {
                    DataValue dv = datas["MixDisp"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1#累计重量
        /// </summary>
        public float MixCum_1
        {
            get
            {
                if (datas.ContainsKey("MixCum_1"))
                {
                    DataValue dv = datas["MixCum_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2#累计重量
        /// </summary>
        public float MixCum_2
        {
            get
            {
                if (datas.ContainsKey("MixCum_2"))
                {
                    DataValue dv = datas["MixCum_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3#累计重量
        /// </summary>
        public float MixCum_3
        {
            get
            {
                if (datas.ContainsKey("MixCum_3"))
                {
                    DataValue dv = datas["MixCum_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4#累计重量
        /// </summary>
        public float MixCum_4
        {
            get
            {
                if (datas.ContainsKey("MixCum_4"))
                {
                    DataValue dv = datas["MixCum_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5#累计重量
        /// </summary>
        public float MixCum_5
        {
            get
            {
                if (datas.ContainsKey("MixCum_5"))
                {
                    DataValue dv = datas["MixCum_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1#累计百分比
        /// </summary>
        public float MixCumPercent_1
        {
            get
            {
                if (datas.ContainsKey("MixCumPercent_1"))
                {
                    DataValue dv = datas["MixCumPercent_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2#累计百分比
        /// </summary>
        public float MixCumPercent_2
        {
            get
            {
                if (datas.ContainsKey("MixCumPercent_2"))
                {
                    DataValue dv = datas["MixCumPercent_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3#累计百分比
        /// </summary>
        public float MixCumPercent_3
        {
            get
            {
                if (datas.ContainsKey("MixCumPercent_3"))
                {
                    DataValue dv = datas["MixCumPercent_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4#累计百分比
        /// </summary>
        public float MixCumPercent_4
        {
            get
            {
                if (datas.ContainsKey("MixCumPercent_4"))
                {
                    DataValue dv = datas["MixCumPercent_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5#累计百分比
        /// </summary>
        public float MixCumPercent_5
        {
            get
            {
                if (datas.ContainsKey("MixCumPercent_5"))
                {
                    DataValue dv = datas["MixCumPercent_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 产量清零
        /// </summary>
        public bool ClearProduction
        {
            get
            {
                if (datas.ContainsKey("ClearProduction"))
                {
                    DataValue dv = datas["ClearProduction"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 流量方式
        /// </summary>
        public bool FlowCalMethodIsA
        {
            get
            {
                if (datas.ContainsKey("FlowCalMethodIsA"))
                {
                    DataValue dv = datas["FlowCalMethodIsA"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 流量反馈
        /// </summary>
        public float FlowDisp
        {
            get
            {
                if (datas.ContainsKey("FlowDisp"))
                {
                    DataValue dv = datas["FlowDisp"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 滤波系数设定
        /// </summary>
        public float FilterParam
        {
            get
            {
                if (datas.ContainsKey("FilterParam"))
                {
                    DataValue dv = datas["FilterParam"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 切换级别设定
        /// </summary>
        public float SwitchLv
        {
            get
            {
                if (datas.ContainsKey("SwitchLv"))
                {
                    DataValue dv = datas["SwitchLv"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 升降增益设定
        /// </summary>
        public float UpDownGain
        {
            get
            {
                if (datas.ContainsKey("UpDownGain"))
                {
                    DataValue dv = datas["UpDownGain"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 比例增益设定
        /// </summary>
        public float PGain
        {
            get
            {
                if (datas.ContainsKey("PGain"))
                {
                    DataValue dv = datas["PGain"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 积分时间设定
        /// </summary>
        public float ITime
        {
            get
            {
                if (datas.ContainsKey("ITime"))
                {
                    DataValue dv = datas["ITime"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 调节系数设定
        /// </summary>
        public float AdjustFactor
        {
            get
            {
                if (datas.ContainsKey("AdjustFactor"))
                {
                    DataValue dv = datas["AdjustFactor"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 调节上限设定
        /// </summary>
        public float AdjustUpper
        {
            get
            {
                if (datas.ContainsKey("AdjustUpper"))
                {
                    DataValue dv = datas["AdjustUpper"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 调节下限设定
        /// </summary>
        public float AdjustLower
        {
            get
            {
                if (datas.ContainsKey("AdjustLower"))
                {
                    DataValue dv = datas["AdjustLower"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 稳定级别切换
        /// </summary>
        public float StableLvSwitch
        {
            get
            {
                if (datas.ContainsKey("StableLvSwitch"))
                {
                    DataValue dv = datas["StableLvSwitch"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// PID调节限幅
        /// </summary>
        public float PIDLimit
        {
            get
            {
                if (datas.ContainsKey("PIDLimit"))
                {
                    DataValue dv = datas["PIDLimit"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 采样时间
        /// </summary>
        public float SampleTime
        {
            get
            {
                if (datas.ContainsKey("SampleTime"))
                {
                    DataValue dv = datas["SampleTime"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// PID结果
        /// </summary>
        public float PIDResult
        {
            get
            {
                if (datas.ContainsKey("PIDResult"))
                {
                    DataValue dv = datas["PIDResult"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// PID调节
        /// </summary>
        public float PIDAdjust
        {
            get
            {
                if (datas.ContainsKey("PIDAdjust"))
                {
                    DataValue dv = datas["PIDAdjust"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 实际速度
        /// </summary>
        public float ScrewCurrentSpeed
        {
            get
            {
                if (datas.ContainsKey("ScrewCurrentSpeed"))
                {
                    DataValue dv = datas["ScrewCurrentSpeed"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 比例增益
        /// </summary>
        public float PGainDisp
        {
            get
            {
                if (datas.ContainsKey("PGainDisp"))
                {
                    DataValue dv = datas["PGainDisp"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 卸料时间
        /// </summary>
        public float UnloadingTime
        {
            get
            {
                if (datas.ContainsKey("UnloadingTime"))
                {
                    DataValue dv = datas["UnloadingTime"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 零位稳定时间
        /// </summary>
        public float ZeroStableTime
        {
            get
            {
                if (datas.ContainsKey("ZeroStableTime"))
                {
                    DataValue dv = datas["ZeroStableTime"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 流量切换允许
        /// </summary>
        public float FlowSwitchEnable
        {
            get
            {
                if (datas.ContainsKey("FlowSwitchEnable"))
                {
                    DataValue dv = datas["FlowSwitchEnable"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 取样时间设定(s)
        /// </summary>
        public float SampleTimeSet
        {
            get
            {
                if (datas.ContainsKey("SampleTimeSet"))
                {
                    DataValue dv = datas["SampleTimeSet"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 额定频率(Hz)
        /// </summary>
        public float ScrewMotorRatedFreq
        {
            get
            {
                if (datas.ContainsKey("ScrewMotorRatedFreq"))
                {
                    DataValue dv = datas["ScrewMotorRatedFreq"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 重量稳定时间(s)
        /// </summary>
        public float WeightStableTime
        {
            get
            {
                if (datas.ContainsKey("WeightStableTime"))
                {
                    DataValue dv = datas["WeightStableTime"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 料位下限设定(kg)
        /// </summary>
        public float MaterialLowLimitSet
        {
            get
            {
                if (datas.ContainsKey("MaterialLowLimitSet"))
                {
                    DataValue dv = datas["MaterialLowLimitSet"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 报警偏差设定(%)
        /// </summary>
        public float AlarmDeviationSet
        {
            get
            {
                if (datas.ContainsKey("AlarmDeviationSet"))
                {
                    DataValue dv = datas["AlarmDeviationSet"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 重量校正系数
        /// </summary>
        public float WeightCorrectFactor
        {
            get
            {
                if (datas.ContainsKey("WeightCorrectFactor"))
                {
                    DataValue dv = datas["WeightCorrectFactor"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 缺料报警设定(kg)
        /// </summary>
        public float LackErrorSet
        {
            get
            {
                if (datas.ContainsKey("LackErrorSet"))
                {
                    DataValue dv = datas["LackErrorSet"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 搅拌时间设定(s)
        /// </summary>
        public float MixingTimeSet
        {
            get
            {
                if (datas.ContainsKey("MixingTimeSet"))
                {
                    DataValue dv = datas["MixingTimeSet"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1次补料偏差
        /// </summary>
        public float FeedingDeviation_1
        {
            get
            {
                if (datas.ContainsKey("FeedingDeviation_1"))
                {
                    DataValue dv = datas["FeedingDeviation_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2次补料偏差
        /// </summary>
        public float FeedingDeviation_2
        {
            get
            {
                if (datas.ContainsKey("FeedingDeviation_2"))
                {
                    DataValue dv = datas["FeedingDeviation_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3次补料偏差
        /// </summary>
        public float FeedingDeviation_3
        {
            get
            {
                if (datas.ContainsKey("FeedingDeviation_3"))
                {
                    DataValue dv = datas["FeedingDeviation_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4次补料偏差
        /// </summary>
        public float FeedingDeviation_4
        {
            get
            {
                if (datas.ContainsKey("FeedingDeviation_4"))
                {
                    DataValue dv = datas["FeedingDeviation_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5次补料偏差
        /// </summary>
        public float FeedingDeviation_5
        {
            get
            {
                if (datas.ContainsKey("FeedingDeviation_5"))
                {
                    DataValue dv = datas["FeedingDeviation_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1次加料设定
        /// </summary>
        public float FeedSet_1
        {
            get
            {
                if (datas.ContainsKey("FeedSet_1"))
                {
                    DataValue dv = datas["FeedSet_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2次加料设定
        /// </summary>
        public float FeedSet_2
        {
            get
            {
                if (datas.ContainsKey("FeedSet_2"))
                {
                    DataValue dv = datas["FeedSet_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3次加料设定
        /// </summary>
        public float FeedSet_3
        {
            get
            {
                if (datas.ContainsKey("FeedSet_3"))
                {
                    DataValue dv = datas["FeedSet_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4次加料设定
        /// </summary>
        public float FeedSet_4
        {
            get
            {
                if (datas.ContainsKey("FeedSet_4"))
                {
                    DataValue dv = datas["FeedSet_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5次加料设定
        /// </summary>
        public float FeedSet_5
        {
            get
            {
                if (datas.ContainsKey("FeedSet_5"))
                {
                    DataValue dv = datas["FeedSet_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1#初始时间
        /// </summary>
        public float InitTime_1
        {
            get
            {
                if (datas.ContainsKey("InitTime_1"))
                {
                    DataValue dv = datas["InitTime_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2#初始时间
        /// </summary>
        public float InitTime_2
        {
            get
            {
                if (datas.ContainsKey("InitTime_2"))
                {
                    DataValue dv = datas["InitTime_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3#初始时间
        /// </summary>
        public float InitTime_3
        {
            get
            {
                if (datas.ContainsKey("InitTime_3"))
                {
                    DataValue dv = datas["InitTime_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4#初始时间
        /// </summary>
        public float InitTime_4
        {
            get
            {
                if (datas.ContainsKey("InitTime_4"))
                {
                    DataValue dv = datas["InitTime_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5#初始时间
        /// </summary>
        public float InitTime_5
        {
            get
            {
                if (datas.ContainsKey("InitTime_5"))
                {
                    DataValue dv = datas["InitTime_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1#最小时间
        /// </summary>
        public float MinTime_1
        {
            get
            {
                if (datas.ContainsKey("MinTime_1"))
                {
                    DataValue dv = datas["MinTime_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2#最小时间
        /// </summary>
        public float MinTime_2
        {
            get
            {
                if (datas.ContainsKey("MinTime_2"))
                {
                    DataValue dv = datas["MinTime_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3#最小时间
        /// </summary>
        public float MinTime_3
        {
            get
            {
                if (datas.ContainsKey("MinTime_3"))
                {
                    DataValue dv = datas["MinTime_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4#最小时间
        /// </summary>
        public float MinTime_4
        {
            get
            {
                if (datas.ContainsKey("MinTime_4"))
                {
                    DataValue dv = datas["MinTime_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5#最小时间
        /// </summary>
        public float MinTime_5
        {
            get
            {
                if (datas.ContainsKey("MinTime_5"))
                {
                    DataValue dv = datas["MinTime_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1#稳重时间
        /// </summary>
        public float StableTime_1
        {
            get
            {
                if (datas.ContainsKey("StableTime_1"))
                {
                    DataValue dv = datas["StableTime_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2#稳重时间
        /// </summary>
        public float StableTime_2
        {
            get
            {
                if (datas.ContainsKey("StableTime_2"))
                {
                    DataValue dv = datas["StableTime_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3#稳重时间
        /// </summary>
        public float StableTime_3
        {
            get
            {
                if (datas.ContainsKey("StableTime_3"))
                {
                    DataValue dv = datas["StableTime_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4#稳重时间
        /// </summary>
        public float StableTime_4
        {
            get
            {
                if (datas.ContainsKey("StableTime_4"))
                {
                    DataValue dv = datas["StableTime_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5#稳重时间
        /// </summary>
        public float StableTime_5
        {
            get
            {
                if (datas.ContainsKey("StableTime_5"))
                {
                    DataValue dv = datas["StableTime_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 1次料仓报警设定
        /// </summary>
        public float BinAlarmSet_1
        {
            get
            {
                if (datas.ContainsKey("BinAlarmSet_1"))
                {
                    DataValue dv = datas["BinAlarmSet_1"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 2次料仓报警设定
        /// </summary>
        public float BinAlarmSet_2
        {
            get
            {
                if (datas.ContainsKey("BinAlarmSet_2"))
                {
                    DataValue dv = datas["BinAlarmSet_2"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 3次料仓报警设定
        /// </summary>
        public float BinAlarmSet_3
        {
            get
            {
                if (datas.ContainsKey("BinAlarmSet_3"))
                {
                    DataValue dv = datas["BinAlarmSet_3"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 4次料仓报警设定
        /// </summary>
        public float BinAlarmSet_4
        {
            get
            {
                if (datas.ContainsKey("BinAlarmSet_4"))
                {
                    DataValue dv = datas["BinAlarmSet_4"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 5次料仓报警设定
        /// </summary>
        public float BinAlarmSet_5
        {
            get
            {
                if (datas.ContainsKey("BinAlarmSet_5"))
                {
                    DataValue dv = datas["BinAlarmSet_5"];
                    if (dv.readVal == null)
                        return 0;
                    else
                        return (float)dv.readVal;
                }
                return 0;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// No.1料仓缺料报警
        /// </summary>
        public bool IsErrorOfLack_1
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfLack_1"))
                {
                    DataValue dv = datas["IsErrorOfLack_1"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// No.2料仓缺料报警
        /// </summary>
        public bool IsErrorOfLack_2
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfLack_2"))
                {
                    DataValue dv = datas["IsErrorOfLack_2"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// No.3料仓缺料报警
        /// </summary>
        public bool IsErrorOfLack_3
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfLack_3"))
                {
                    DataValue dv = datas["IsErrorOfLack_3"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// No.4料仓缺料报警
        /// </summary>
        public bool IsErrorOfLack_4
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfLack_4"))
                {
                    DataValue dv = datas["IsErrorOfLack_4"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// No.5料仓缺料报警
        /// </summary>
        public bool IsErrorOfLack_5
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfLack_5"))
                {
                    DataValue dv = datas["IsErrorOfLack_5"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// No.1料斗加料异常
        /// </summary>
        public bool IsErrorOfAdd_1
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfAdd_1"))
                {
                    DataValue dv = datas["IsErrorOfAdd_1"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// No.2料斗加料异常
        /// </summary>
        public bool IsErrorOfAdd_2
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfAdd_2"))
                {
                    DataValue dv = datas["IsErrorOfAdd_2"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// No.3料斗加料异常
        /// </summary>
        public bool IsErrorOfAdd_3
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfAdd_3"))
                {
                    DataValue dv = datas["IsErrorOfAdd_3"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// No.4料斗加料异常
        /// </summary>
        public bool IsErrorOfAdd_4
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfAdd_4"))
                {
                    DataValue dv = datas["IsErrorOfAdd_4"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// No.5料斗加料异常
        /// </summary>
        public bool IsErrorOfAdd_5
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfAdd_5"))
                {
                    DataValue dv = datas["IsErrorOfAdd_5"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 螺杆缺料报警
        /// </summary>
        public bool IsErrorOfScrewLack
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfScrewLack"))
                {
                    DataValue dv = datas["IsErrorOfScrewLack"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 螺杆流量超偏差报警
        /// </summary>
        public bool IsErrorOfScrewFlow
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfScrewFlow"))
                {
                    DataValue dv = datas["IsErrorOfScrewFlow"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 搅拌电机报警
        /// </summary>
        public bool IsErrorOfBlender
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfBlender"))
                {
                    DataValue dv = datas["IsErrorOfBlender"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 急停开启
        /// </summary>
        public bool IsErrorOfScram
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfScram"))
                {
                    DataValue dv = datas["IsErrorOfScram"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }

        /// <summary>
        /// 搅拌电机急停开启
        /// </summary>
        public bool IsErrorOfBlender2
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfBlender2"))
                {
                    DataValue dv = datas["IsErrorOfBlender2"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }


        ///搅拌电机故障 
        public bool IsErrorOfMixerMotor
        {
            get
            {
                if (datas.ContainsKey("IsErrorOfMixerMotor"))
                {
                    DataValue dv = datas["IsErrorOfMixerMotor"];
                    if (dv.readVal == null)
                        return false;
                    else
                        return (bool)dv.readVal;
                }
                return false;
            }
            set
            {
                SetProperValue(value);
            }
        }
        #endregion


        #region 混料料仓队列
        private List<IngredientCell> ingredients = new List<IngredientCell>();

        public List<IngredientCell> Ingredients
        {
            get
            {
                return ingredients;
            }
        }

        /// <summary>
        /// 料仓，混料参数 绑定到 对应的队列
        /// </summary>
        void BinsBinding()
        {
            Ingredients.Clear();

            for (int i = 0; i < BinCnt; i++)
            {
                int no = i + 1;
                IngredientCell ingredient = new IngredientCell();
                ingredient.Number = no.ToString();
                Misc.BindingOperations.SetBinding(this, "MixPSet_" + no.ToString(), ingredient, "MixPSet", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "MixPDisp_" + no.ToString(), ingredient, "MixPDisp", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "MixCumPercent_" + no.ToString(), ingredient, "MixCumPercent", Misc.BindingOperations.BindingMode.TwoWay);

                Misc.BindingOperations.SetBinding(this, "MixSet_" + no.ToString(), ingredient, "MixSet", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "MixDisp_" + no.ToString(), ingredient, "MixDisp", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "MixCum_" + no.ToString(), ingredient, "MixCum", Misc.BindingOperations.BindingMode.TwoWay);

                Ingredients.Add(ingredient);
            }

            IngredientParams.Clear();

            for (int i = 0; i < BinCnt; i++)
            {
                int no = i + 1;
                IngredientParamCell ingredientParam = new IngredientParamCell();
                ingredientParam.Number = no.ToString();
                Misc.BindingOperations.SetBinding(this, "FeedingDeviation_" + no.ToString(), ingredientParam, "FeedingDeviation", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "FeedSet_" + no.ToString(), ingredientParam, "FeedSet", Misc.BindingOperations.BindingMode.TwoWay);

                Misc.BindingOperations.SetBinding(this, "InitTime_" + no.ToString(), ingredientParam, "InitTime", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "MinTime_" + no.ToString(), ingredientParam, "MinTime", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "StableTime_" + no.ToString(), ingredientParam, "StableTime", Misc.BindingOperations.BindingMode.TwoWay);
                Misc.BindingOperations.SetBinding(this, "BinAlarmSet_" + no.ToString(), ingredientParam, "BinAlarmSet", Misc.BindingOperations.BindingMode.TwoWay);

                IngredientParams.Add(ingredientParam);
            }
        }
        #endregion
        #region 混料参数队列
        private List<IngredientParamCell> ingredientParams = new List<IngredientParamCell>();

        public List<IngredientParamCell> IngredientParams
        {
            get
            {
                return ingredientParams;
            }
        }
        #endregion
    }

    public class IngredientCell : INotifyPropertyChanged
    {
        /// <summary>
        /// 颜色
        /// </summary>
        public Brush Color { get; set; }


        /// <summary>
        /// 序号
        /// </summary>
        public string Number { get; set; }


        private bool mixLight;
        /// <summary>
        /// 混料 进行中
        /// </summary>
        public bool MixLight
        {
            get { return mixLight; }
            set
            {
                if (mixLight != value)
                {
                    mixLight = value;
                    NotifyPropertyChanged("MixLight");
                }
            }
        }
        private float mixPSet;
        /// <summary>
        /// 比例设定
        /// </summary>
        public float MixPSet
        {
            get { return mixPSet; }
            set
            {
                if (mixPSet != value)
                {
                    mixPSet = value;
                    NotifyPropertyChanged("MixPSet");
                }
            }
        }

        private float mixPDisp;
        /// <summary>
        /// 比例显示
        /// </summary>
        public float MixPDisp
        {
            get { return mixPDisp; }
            set
            {
                if (mixPDisp != value)
                {
                    mixPDisp = value;
                    NotifyPropertyChanged("MixPDisp");
                }
            }
        }


        private float mixCumPercent;
        /// <summary>
        /// 累计重量比例
        /// </summary>
        public float MixCumPercent
        {
            get { return mixCumPercent; }
            set
            {
                if (mixCumPercent != value)
                {
                    mixCumPercent = value;
                    NotifyPropertyChanged("MixCumPercent");
                }
            }
        }
        private float mixset;
        /// <summary>
        /// 重量设定
        /// </summary>
        public float MixSet
        {
            get { return mixset; }
            set
            {
                if (mixset != value)
                {
                    mixset = value;
                    NotifyPropertyChanged("MixSet");
                }
            }
        }

        private float mixdisp;
        /// <summary>
        /// 重量显示
        /// </summary>
        public float MixDisp
        {
            get { return mixdisp; }
            set
            {
                if (mixdisp != value)
                {
                    mixdisp = value;
                    NotifyPropertyChanged("MixDisp");
                }
            }
        }

        private float mixcum;
        /// <summary>
        /// 累计重量
        /// </summary>
        public float MixCum
        {
            get { return mixcum; }
            set
            {
                if (mixcum != value)
                {
                    mixcum = value;
                    NotifyPropertyChanged("MixCum");
                }
            }
        }


        public IngredientCell()
        {

        }

        protected void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class IngredientParamCell : INotifyPropertyChanged
    {
        /// <summary>
        /// 序号
        /// </summary>
        public string Number { get; set; }


        private float feedingDeviation;
        /// <summary>
        /// 补料偏差
        /// </summary>
        public float FeedingDeviation
        {
            get { return feedingDeviation; }
            set
            {
                if (feedingDeviation != value)
                {
                    feedingDeviation = value;
                    NotifyPropertyChanged("FeedingDeviation");
                }
            }
        }

        private float feedSet;
        /// <summary>
        /// 加料设定
        /// </summary>
        public float FeedSet
        {
            get { return feedSet; }
            set
            {
                if (feedSet != value)
                {
                    feedSet = value;
                    NotifyPropertyChanged("FeedSet");
                }
            }
        }


        private float initTime;
        /// <summary>
        /// 初始时间
        /// </summary>
        public float InitTime
        {
            get { return initTime; }
            set
            {
                if (initTime != value)
                {
                    initTime = value;
                    NotifyPropertyChanged("InitTime");
                }
            }
        }
        private float minTime;
        /// <summary>
        /// 最小时间
        /// </summary>
        public float MinTime
        {
            get { return minTime; }
            set
            {
                if (minTime != value)
                {
                    minTime = value;
                    NotifyPropertyChanged("MinTime");
                }
            }
        }

        private float stableTime;
        /// <summary>
        /// 稳重时间
        /// </summary>
        public float StableTime
        {
            get { return stableTime; }
            set
            {
                if (stableTime != value)
                {
                    stableTime = value;
                    NotifyPropertyChanged("StableTime");
                }
            }
        }

        private float binAlarmSet;
        /// <summary>
        /// 料仓报警设定
        /// </summary>
        public float BinAlarmSet
        {
            get { return binAlarmSet; }
            set
            {
                if (binAlarmSet != value)
                {
                    binAlarmSet = value;
                    NotifyPropertyChanged("BinAlarmSet");
                }
            }
        }


        public IngredientParamCell()
        {

        }

        protected void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
    }

}
