﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DocumentFormat.OpenXml;
using DocumentFormat.OpenXml.Spreadsheet;
using Excel_Access;
using OfficeOpenXml;
using System.ComponentModel;
using System.Text.RegularExpressions;

namespace Autogen_WS
{
    public class DataInfos:INotifyPropertyChanged
    {
        
        class ItemInfo
        {
            /// <summary>
            /// 属性名
            /// </summary>
            public string propName;
            /// <summary>
            /// 属性描述
            /// </summary>
            public string propDescription;

            /// <summary>
            /// plc 序号
            /// </summary>
            public int plc_index;
            /// <summary>
            /// 寄存器类型 0,1,3,4
            /// </summary>
            public string reg_type;
            /// <summary>
            /// 地址
            /// </summary>
            public int mb_addr=-1;

            /// <summary>
            /// 数据类型， 与寄存器类型不同, bool, float, UInt16
            /// </summary>
            public string typeName;
            /// <summary>
            /// 放大
            /// </summary>
            public double scale=1;

            /// <summary>
            /// 类名
            /// </summary>
            public string weighterName;

            public ItemInfo()
            { }

            public ItemInfo Clone() 
            {
                ItemInfo ii = new ItemInfo();
                ii.propName = propName;
                ii.propDescription = propDescription;
                ii.plc_index = plc_index;
                ii.reg_type = reg_type;
                ii.mb_addr = mb_addr;
                ii.typeName = typeName;
                ii.scale = scale;
                ii.weighterName = weighterName;
                return ii;
            }

            public string GetPropertyType() 
            {
                if (typeName == "bool")
                    return "bool";
                else
                    return "float";
            }
        }
        List<ItemInfo> infos = new List<ItemInfo>();

        DeviceAddr mDeviceAddrs = new DeviceAddr();
        DeviceTranslationTable mDeviceTranslationTable = new DeviceTranslationTable();
        WeightInfo mWeightInfos = new WeightInfo();
        int plc_cnt=0;
        int weight_cnt=0;


        private double progress = 0;
        public double Progress 
        {
            get {
                return progress;
            }
            set {
                if (progress != value) 
                {
                    progress = value;
                    NotifyPropertyChanged("Progress");
                }
            }
        }
        private string xmlfile;
        public string XmlFile
        {
            get
            {
                return xmlfile;
            }
            set
            {
                if (xmlfile != value)
                {
                    xmlfile = value;
                    NotifyPropertyChanged("XmlFile");
                }
            }
        }

        private bool gen_weighterc;
        public bool GenWeighterC
        {
            get
            {
                return gen_weighterc;
            }
            set
            {
                if (gen_weighterc != value)
                {
                    gen_weighterc = value;
                    NotifyPropertyChanged("GenWeighterC");
                }
            }
        }

        private bool gen_ws;
        public bool GenWS
        {
            get
            {
                return gen_ws;
            }
            set
            {
                if (gen_ws != value)
                {
                    gen_ws = value;
                    NotifyPropertyChanged("GenWS");
                }
            }
        }

        private string csharppath;
        public string CSharpPath
        {
            get { return csharppath; }
            set
            {
                if (csharppath != value)
                {
                    csharppath = value;
                    NotifyPropertyChanged("CSharpPath");
                }
            }
        }

        public DataInfos()
        {

        }
        //public DataInfos(string xmlfile)
        //{
        //    LoadXmlFile(xmlfile);
        //}

        private bool NextCell(int max, ref int iCol, ref int iRow)
        {
            if (iRow == 1)
            {
                if (iCol + 1 > max) return false;
                iRow = iCol + 1;
                iCol = 1;
                return true;
            }
            else if (iCol < iRow)
            {
                iCol++;
                return true;
            }
            iRow--;
            return false;
        }
        public bool LoadXmlFile() 
        {
            return LoadXmlFile(XmlFile);
        }

        /// <summary>
        /// 分析 "设备1_读写4WUB3821" 分拆为plc序号，寄存器类型, 寄存器地址
        /// </summary>
        /// <param name="plc_addr">寄存器地址字符串</param>
        /// <param name="plc_index">plc序号</param>
        /// <param name="type">寄存器类型</param>
        /// <param name="addr">寄存器地址</param>
        /// <returns>是否成功</returns>
        bool AnalysisAddr(string plc_addr, out int plc_index, out string type, out int addr)
        {
            plc_index = -1;
            type = null;
            addr = 0;
            //string plc_addr = "设备1_读写4WUB3821";
            //Console.WriteLine("查找对象=" + plc_addr);
            Regex r = new Regex(@"设备(\d)_(\w+)");
            Match m = r.Match(plc_addr);
            if (!m.Success)
                return false;
            
            if (!int.TryParse(m.Groups[1].Value, out plc_index))
                return false;

            //查表
            var cells = from _cell in mDeviceTranslationTable.Tables where _cell.VariableName == m.Groups[2].Value select _cell;
            if (cells.Count() == 0)
                return false;

            type = cells.First().Mode;
            addr = cells.First().Addr;

            
            return true;

        }


        /// <summary>
        /// 加载 HMI-53435最后确定.xlsm
        /// </summary>
        /// <param name="xmlfile"></param>
        /// <returns></returns>
        public bool LoadXmlFile(string xmlfile)
        {
            //加载地址
            mDeviceAddrs.Load(xmlfile);
            //加载转换表
            mDeviceTranslationTable.Load(xmlfile);

            mWeightInfos.Load(xmlfile);

            int iCol;
            int iRow;


            int i_propDescription = -1;//描述
            int i_propName = -1;//属性名
            int i_typeName = -1;//类型
            int i_scale = -1;//放大
            int i_mb_addr = -1;
            int weighter_cnt = 0;//

            string str;

            infos.Clear();
            Progress = 0;
            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xmlfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets["各层参数"])
                {
                    iCol = 1;
                    iRow = 1;


                    // 寻找每项的列号i_XXXX
                    int i = iCol;

                    Regex r_header_addr = new Regex(@"[A-F]层HMI寄存器");

                    while (true)
                    {
                        try
                        {
                            str = sheet.Cells[iRow, i].Value.ToString();
                        }
                        catch
                        {
                            break;
                        }
                        if (str == null) break;
                        if (str == "") break;
                        if (str.Contains("描述"))
                            i_propDescription = i;
                        else if (str.Contains("属性名"))
                            i_propName = i;
                        else if (str.Contains("类型"))
                            i_typeName = i;
                        else if (str.Contains("放大"))
                            i_scale = i;
                        else
                        {
                            if (r_header_addr.IsMatch(str))
                            {
                                if (i_mb_addr == -1)//还没找到HMI寄存器
                                    i_mb_addr = i;
                                weighter_cnt++;
                            }
                        }

                        i++;
                        continue;
                    }



                    if ((i_propDescription < 0) ||
                        (i_propName < 0) ||
                        (i_typeName < 0) ||
                        (i_scale < 0) ||
                        (i_mb_addr < 0))
                        return false;





                    weight_cnt = weighter_cnt;

                    plc_cnt = 0;
                    // 生成C#对象属性列表
                    iRow++;
                    while (true)
                    {
                        ItemInfo ii = new ItemInfo();

                        try
                        {
                            ii.propDescription = sheet.Cells[iRow, i_propDescription].Value.ToString();
                        }
                        catch
                        {
                            ii.propDescription = null;
                        }


                        try
                        {
                            ii.propName = sheet.Cells[iRow, i_propName].Value.ToString();
                        }
                        catch
                        {
                            ii.propName = null;//没有名称异常
                            break;
                        }



                        try
                        {
                            ii.typeName = sheet.Cells[iRow, i_typeName].Value.ToString();
                        }
                        catch
                        {
                            ii.typeName = null;//没有类型异常
                            break;
                        }


                        try
                        {
                            string scale_str = sheet.Cells[iRow, i_scale].Value.ToString();
                            if (!string.IsNullOrEmpty(scale_str))
                            {
                                double d;
                                if (double.TryParse(scale_str, out d))
                                {
                                    ii.scale = d;
                                }
                            }
                        }
                        catch
                        {

                        }

                        //propertyname 不能有重复
                        if (!infos.Any((info) => { return info.propName == ii.propName; }))
                        {
                            for (int wi = 0; wi < weighter_cnt; wi++)
                            {
                                ii.weighterName = new string(new char[] { (char)('A' + wi) });
                                string plc_addr;
                                try
                                {
                                    plc_addr = sheet.Cells[iRow, i_mb_addr + wi].Value.ToString();
                                }
                                catch
                                {
                                    //空的
                                    continue;
                                }

                                if (!AnalysisAddr(plc_addr, out ii.plc_index, out ii.reg_type, out ii.mb_addr))
                                    continue;
                                if ((ii.plc_index + 1) > plc_cnt)
                                    plc_cnt = ii.plc_index + 1;

                                infos.Add(ii);

                                ii = ii.Clone();
                            }
                        }
                        iRow++;
                        Progress += 1;
                        if (Progress >= 100)
                            Progress = 50;
                    }
                    Progress = 100;
                    return true;
                }
            }
        }

        public void Gen_IWeighterC(StreamWriter sw)
        {
            Dictionary<string, ItemInfo> d = new Dictionary<string, ItemInfo>();

            foreach (ItemInfo ii in infos)
            {
                if (d.ContainsKey(ii.propName)) continue;
                d.Add(ii.propName, ii);
            }

            sw.WriteLine("        #region 辅助代码生成");
            sw.WriteLine();
            foreach (KeyValuePair<string, ItemInfo> sii in d)
            {
                ItemInfo info = sii.Value;
                sw.WriteLine("        /// <summary>");
                sw.WriteLine("        /// {0}", info.propDescription);
                sw.WriteLine("        /// </summary>");
                sw.WriteLine("        {0} {1}", info.GetPropertyType(), info.propName);
                sw.WriteLine("        {");
                sw.WriteLine("            get;");
                sw.WriteLine("            set;");
                sw.WriteLine("        }");
                sw.WriteLine();
            }
            sw.WriteLine("        #endregion");
        }

        void Gen_WeighterC(StreamWriter sw)
        {
           

            Dictionary<string, ItemInfo> d = new Dictionary<string, ItemInfo>();

            foreach (ItemInfo ii in infos)
            {
                if (d.ContainsKey(ii.propName)) continue;
                d.Add(ii.propName, ii);
            }
            sw.WriteLine("        #region 辅助代码生成");
            foreach (KeyValuePair<string, ItemInfo> sii in d)
            {
                ItemInfo info = sii.Value;
                sw.WriteLine("        /// <summary>");
                sw.WriteLine("        /// {0}", info.propDescription);
                sw.WriteLine("        /// </summary>");
                sw.WriteLine("        public {0} {1}", info.GetPropertyType(), info.propName);
                sw.WriteLine("        {");
                sw.WriteLine("            get");
                sw.WriteLine("            {");
                sw.WriteLine("                if (datas.ContainsKey(\"{0}\"))", info.propName);
                sw.WriteLine("                {");
                sw.WriteLine("                    DataValue dv = datas[\"{0}\"];", info.propName);
                sw.WriteLine("                    if (dv.readVal == null)");

                if (info.GetPropertyType() == "bool")
                    sw.WriteLine("                        return false;");
                else
                    sw.WriteLine("                        return 0;");

                sw.WriteLine("                    else");
                sw.WriteLine("                        return ({0})dv.readVal;", info.GetPropertyType());
                sw.WriteLine("                }");

                if (info.GetPropertyType() == "bool")
                    sw.WriteLine("                return false;");
                else
                    sw.WriteLine("                return 0;");

                sw.WriteLine("            }");
                sw.WriteLine("            set");
                sw.WriteLine("            {");
                sw.WriteLine("                SetProperValue(value);");
                sw.WriteLine("            }");
                sw.WriteLine("        }");
                sw.WriteLine();
            }
            sw.WriteLine("        #endregion");
        }

        public void Gen_WeighterC()
        {
            //找到 #region 辅助代码生成 与最接近的 #endregion

            //StringBuilder sb = new StringBuilder();
            List<String> file_content_header = new List<string>();
            List<String> file_content_tailer = new List<string>();
            int state = 0;
            int region_cnt = 0;

            string weightc_new_path = Path.Combine(Path.GetDirectoryName(XmlFile), "Generated", Path.GetFileName(CSharpPath));

            StreamWriter sw = new StreamWriter(weightc_new_path, false, Encoding.UTF8);
            StreamReader sr = new StreamReader(CSharpPath);

            //把整个文件读到 sb中
            while (!sr.EndOfStream)
            {
                string line = sr.ReadLine();
                switch (state)
                {
                    case 0:
                        if (line.Contains("#region 辅助代码生成"))
                        {
                            //找到了
                            state = 1;
                            region_cnt = 1;
                        }
                        else
                        {
                            sw.WriteLine(line);
                        }
                        break;
                    case 1://跳过中间的
                        if (line.Contains("#region"))
                        {
                            region_cnt++;
                        }
                        else if (line.Contains("#endregion"))
                        {
                            region_cnt--;
                            if (region_cnt == 0)
                            {
                                //插入 自动代码
                                Gen_WeighterC(sw);
                                
                                //下面都是尾
                                state = 2;
                            }
                        }
                        break;

                    case 2://记录尾巴
                        sw.WriteLine(line);
                        break;
                }
            }
            sr.Close();
            sw.Flush();
            sw.Close();

        }
        public void Gen_WS_XML()
        {
            PLCGroup plcgroup = new PLCGroup();

            for (int i=0;i<plc_cnt && i< mDeviceAddrs.EP.Count(); i++)
            {
                string ep_str = mDeviceAddrs.EP[i];
                plcgroup.Devices.Add(new PLCGroup.PLCDevice() { EP = Misc.StringConverter.ToIPEndPoint(ep_str) });
            }

            plcgroup.Weights.AddRange(mWeightInfos.weights);

            foreach (ItemInfo ii in infos)
            {
                plcgroup.Variables.Add(new PLCGroup.PLCVariable()
                {
                    DeviceIndex = ii.plc_index,
                    Mode = ii.reg_type,
                    Addr = ii.mb_addr,
                    Type = ii.typeName,
                    Scale = ii.scale,
                    OwnerName = ii.weighterName,
                    PropertyName = ii.propName
                });
            }
            string ws_path = Path.Combine(Path.GetDirectoryName(XmlFile),"Generated","WS.xml");
            plcgroup.Save(ws_path);
        }


        protected void NotifyPropertyChanged(string propertyname) 
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
    }
}
